<?php

namespace App\Console\Commands;
use App\Mail\AdviseBroadcastEmail;
use App\Models\Broadcast;
use App\Models\User;
use Illuminate\Console\Command;

class AdviseBroadcast extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'app:advise-broadcast';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'This is to notify the admin regarding the broadcast sent within the day.';

    /**
     * Execute the console command.
     */
    public function handle()
    {
        try {
            $date = date('Y-m-d');
            $adminuser = User::where('id', 1)->where('role', 'admin')->first();
            $email = $adminuser->email;
            $email_count = Broadcast::whereDate('created_at', '<=', $date)
                    ->whereDate('created_at','>=', $date)->where('format', 'email')->count();
            $platform_count = Broadcast::whereDate('created_at', '<=', $date)
            ->whereDate('created_at','>=', $date)->where('format', 'platform')->count();
            $details = [
                'title' => 'Broadcast Statistics from '.env('APP_NAME').'!',
                'appname' => env('APP_NAME'),
                'email' => $email,
                'email_count' => $email_count,
                'platform_count' => $platform_count,
                'total' => $email_count+$platform_count,
            ];
            try {
                \Mail::to($email)->send(new AdviseBroadcastEmail($details));
                $message = "Sending Advise email done.";
                echo $message."\n";
                \Log::info($message);
            } catch (\Exception $e) {
                \Log::error($e);
            }
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
}
