<?php

namespace App\Console\Commands;

use App\Mail\BroadcastEmail;
use App\Models\Broadcast;
use App\Models\Promo;
use App\Models\User;
use Illuminate\Console\Command;

class BroadcastEmails extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'app:broadcast-emails';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Broadcast emails for user the has notifications.';

    /**
     * Execute the console command.
     */
    public function handle()
    {
        try {
            //Find all notification that are within the start and end date range and is an email format
            $date = date('Y-m-d');
            $notifications = Promo::where('format', 'email')
                ->whereDate('start', '<=', $date)
                ->whereDate('end','>=', $date)
                ->where('broadcast', 1)
                // ->limit(2)
                ->get();
            if (! $notifications->isEmpty()) {
                foreach ($notifications->toArray() as $notif) {
                    $users = User::whereNull('deleted_at')->with('profile')->get();
                    foreach ($users as $user) {

                        $broadcastDone = Broadcast::where('user_id', $user->id)->where('notif_id', $notif['id'])->where('format', 'email')->first();
                        if ($user->profile->newsletter > 0 && is_null($broadcastDone)) {
                            $person = !empty($user->profile->firstname) ? $user->profile->firstname : $user->email;
                            $email = $user->email;
                            $details = [
                                'title' => 'Check out '.env('APP_NAME').'!',
                                'appname' => env('APP_NAME'),
                                'person' => $person,
                                'email' => $email,
                                'promo' => $notif
                            ];
                            try {
                                \Mail::to($email)->send(new BroadcastEmail($details));
                                $broadcast = new Broadcast();
                                $broadcast->user_id = $user->id;
                                $broadcast->notif_id = $notif['id'];
                                $broadcast->format = 'email';
                                $broadcast->save();
                            } catch (\Exception $e) {
                                \Log::error($e);
                            }

                        }

                    }
                }

            }

            echo "Email broadcasting is done.\n";
            \Log::info('Email broadcasting is done.');

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
}
