<?php

namespace App\Helpers;

use Illuminate\Support\Str;
use App\Helpers\Utilities;
use App\Models\UserProfile;
use App\Models\User;
use App\Models\Media;

class MediaHelper
{
    public function returnMediaRule($type='img')
    {
        $imgrule = 'required|mimetypes:image/jpeg,image/png,image/jpg,image/gif,image/svg,application/octet-stream';
        switch ($type) {
            case 'vid':
                $imgrule = 'required|mimes:mp4|max:2048';
                break;
            default:
                $imgrule = 'required|mimetypes:image/jpeg,image/png,image/jpg,image/gif,image/svg,application/octet-stream';
                break;
        }
        return $imgrule;
    }

    public function storeItem($location='s3',$data=[], $request=[])
    {
        try {

            $user                = $request->user();
            $file                = $request->file('item');
            $basedir             = $data['type'] == 'vid' ? 'videos' : 'images';
            $filename            = (new Utilities)->generateHash();
            $directory           = $basedir.'/'.md5($user->id);
            if (!\Storage::disk('public')->exists($directory)) {
                \Storage::disk('public')->makeDirectory($directory);
            }

            $generic_url        = $directory.'/'.$filename;
            $format             = $request->item->extension();
            $orig_url           = $generic_url.'.'.$format;
            $thumb_orig_url     = $generic_url.'_thumb.'.$format;
            $convertedOrig      = $this->convertToOptimized(150, $file, $format, $orig_url);
            $convertedThumb     = $this->convertToOptimized(10, $file, $format, $thumb_orig_url, true);
            // $origPath           = \Storage::disk('s3')->put($directory, $file);
            // $origS3Url          = \Storage::disk('s3')->url($origPath);
            //Orig item
            $origcontent        = \Storage::disk('public')->get($orig_url);
            $origS3Url          = '';
            $thumbS3Url         = '';
            if ($location == 's3') {
                $origPath           = \Storage::disk('s3')->put($orig_url, $origcontent);
                $origS3Url          = \Storage::disk('s3')->url($orig_url);
            }

            if ($location == 's3') {
                imagedestroy($convertedOrig);
                \Storage::disk('public')->delete($orig_url);
            }

            //Thumb item
            $origthumb            = \Storage::disk('public')->get($thumb_orig_url);
            if ($location == 's3') {
                $thumbPath          = \Storage::disk('s3')->put($thumb_orig_url, $origthumb);
                $thumbS3Url         = \Storage::disk('s3')->url($thumb_orig_url);
                imagedestroy($convertedThumb);
                \Storage::disk('public')->delete($thumb_orig_url);
            }

            return $location == 's3' ? ['src'=> $origS3Url, 'thumb' => $thumbS3Url] : ['src'=> $orig_url, 'thumb' => $thumb_orig_url];

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }



    public function convertToOptimized($percentage=10, $file='', $format='jpg', $path='', $blur=false)
    {
        try {
            $im = null;
            // \Log::info($file->getClientMimeType());
            if ($file->getClientMimeType() == 'application/octet-stream') {
                $im = imagecreatefromstring(file_get_contents($file));
            } else {
                $im = $format == 'jpg' ? imagecreatefromjpeg($file) : imagecreatefrompng($file);
            }

            $width = imagesx($im);
            $height = imagesy($im);
            $newwidth = $width * ($percentage / 100);
            $newheight = $height * ($percentage / 100);
            $thumb = imagecreatetruecolor($newwidth, $newheight);
            imagecopyresized($thumb, $im, 0, 0, 0, 0, $newwidth, $newheight, $width, $height);
            // $localStorage = \Storage::disk('public')->getDriver()->getAdapter()->getPathPrefix();
            $localStorage = \Storage::disk('public')->path('');
            for ($x=1; $x<=5; $x++) {
                imagefilter($thumb, IMG_FILTER_GAUSSIAN_BLUR, 999);
            }
            $format == 'jpg' ? imagejpeg($thumb, $localStorage.$path) : imagepng($thumb, $localStorage.$path);
            return $thumb;
        } catch(\Exception $e) {
            \Log::error($e);
        }
    }

    public function deleteItemOnS3($item=[])
    {
        $done = false;
        try {
            \Storage::disk('s3')->delete($item->src);
            \Storage::disk('s3')->delete($item->thumb);
            $done = true;
            return $done;
        } catch (\Exception $e) {
            \Log::error($e);
            return $done;
        }
    }

    public function storeAttachmentToS3($data=[], $request=[])
    {
        try {
            $user                = $request->user();
            $file                = $request->file('file');
            $basedir             = 'attachments';
            $filename            = $file->getClientOriginalName();
            $directory           = $basedir.'/'.md5($user->id);
            $orig_url            = $directory.'/'.$filename;

            $origPath            = \Storage::disk('s3')->put($directory, $file);
            $origS3Url           = \Storage::disk('s3')->url($origPath);

            return $origS3Url;

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function attachItemData($media=[])
    {
        try {
            if ($media->item_id_reference == 'user') {
                $target_key = $media->type == 'profile' ? 'avatar' : 'cover';
                $profile = UserProfile::where('user_id', $media->item_id)->first();
                $profile->{$target_key} = $media->src;
                $profile->save();
                $user = User::where('id', $media->item_id)->with('profile')->first();
                $media->item_data = $user;
            }
            return $media;

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
}
