<?php

namespace App\Helpers;

use Illuminate\Support\Str;
use App\Models\User;
use App\Models\UserProfile;
use App\Models\Transaction;
use SimpleSoftwareIO\QrCode\Facades\QrCode;

class UserHelper
{
    public static function deleteUser($data=[], $role='cashier')
    {
        try {
            $user = User::where('id', $data['user_id'])->where('role', $role)->first();
            if (is_null($user)) {
                return ['result' => false, 'msg' => 'User does not exist.'];
            }
            $user->delete();

            $profile = UserProfile::where('user_id', $data['user_id'])->first();
            if (is_null($profile)) {
                return ['result' => false, 'msg' => 'User Profile does not exist.'];
            }
            $profile->delete();
            return ['result' => true,'msg' => 'User data successfully deleted.'];

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
    public static function createUser($data=[])
    {
        try {
            $new_user = User::create(['email'=>$data['email'], 'password' => $data['password']]);
            $username = explode('@',$data['email']);
            $new_user->username = trim($username[0]);
            $new_user->remember_token = Str::random(10);
            $new_user->role = 'admin';
            $new_user->save();
            return $new_user;
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public static function createCashier($data=[], $parent_id=0)
    {
        try {
            $new_user = static::createUser($data);
            $new_user->parent_id = $parent_id;
            $new_user->role = 'cashier';
            return $new_user;
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public static function createCustomer($data=[], $parent_id=0)
    {
        try {
            $new_user = static::createUser($data);
            $new_user->parent_id = $parent_id;
            $new_user->role = 'customer';
            return $new_user;
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public static function generateQrCode($qrstring='', $size=200)
    {
        try {
            return QrCode::size($size)->format(env('QRFORMAT'))->generate($qrstring);
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public static function getLastTransaction($user_id=0)
    {
        try {
            return Transaction::where('user_id', $user_id)->latest()->first();
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public static function getTotalPoints($user_id=0)
    {
        try {
            return number_format(Transaction::where('user_id', $user_id)->where('claimed', 0)->whereNull('deleted_at')->sum('points'));
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
}
