<?php

namespace App\Helpers;

use Illuminate\Support\Str;

class Utilities
{
    public function getBase64Image($url)
    {
    try {
        $image = file_get_contents($url);
        if ($image !== false && !is_null($image)){
        return 'data:image/jpg;base64,'.base64_encode($image);
        }
    } catch (\Exception $e) {
        return "";
    }
    }

    public function fakeImage($format='url', $size='600')
    {
    $id = $this->getRandom(1, 50, ['15','28','35']);
    $url = "https://picsum.photos/id/{$id}/{$size}/{$size}.jpg";
    return $format == ( 'url' ? $url : empty($this->getBase64Image($url)) ) ? $url : $this->getBase64Image($url);
    }

    public function fakeVideo()
    {
    $index = rand(0, 1);
    $viddata = config('media')[$index];
    return $viddata;
    }

    public function getRandom($min=1, $max=10, $except=[])
    {
    $rand = rand($min, $max);
    if (!empty($except) && in_array($rand, $except)) {
        $this->getRandom($min, $max, $except);
    } else {
        return $rand;
    }
    }

    public function generateHash($extra='')
    {
        return md5(microtime() . Str::random() . $extra);
    }

    public function slugify($string, $replace = array(), $delimiter='-') {
    // https://github.com/phalcon/incubator/blob/master/Library/Phalcon/Utils/Slug.php
    if (!extension_loaded('iconv')) {
        throw new Exception('iconv module not loaded');
    }
    // Save the old locale and set the new locale to UTF-8
    $oldLocale = setlocale(LC_ALL, '0');
    setlocale(LC_ALL, 'en_US.UTF-8');
    $clean = iconv('UTF-8', 'ASCII//TRANSLIT', $string);
    if (!empty($replace)) {
        $clean = str_replace((array) $replace, ' ', $clean);
    }
    $clean = preg_replace("/[^a-zA-Z0-9\/_|+ -]/", '', $clean);
    $clean = strtolower($clean);
    $clean = preg_replace("/[\/_|+ -]+/", $delimiter, $clean);
    $clean = trim($clean, $delimiter);
    // Revert back to the old locale
    setlocale(LC_ALL, $oldLocale);
    return $clean;
    }

    public function encdecData($type='en', $data)
    {
    return $type==='de' ? unserialize(urldecode(base64_decode($data))) : base64_encode(urlencode(serialize($data)));
    }

    public function numonly($input)
    {
    return preg_replace('/[^0-9]/', '', $input);
    }

    public static function menuSetter($menulist=[])
    {
        $user = \Auth::user();
        $route = \Route::current()->getName();
        $role = $user->role;
        $user_access = config('menu.role_menus')[$role];
        $new_menu = [];
        // \Log::info("OLD MENU", $menulist);
        for ($i=0; $i < count($menulist); $i++) {

            if ($menulist[$i]['type'] == 'single') {
                //If user access is not allowed remove from array
                if (isset($menulist[$i]) && in_array($menulist[$i]['id'], $user_access)) {
                    if ($menulist[$i]['route'] == $route) {
                        $menulist[$i]['status'] = 'active';
                    }
                    $new_menu[] = $menulist[$i];
                }
            }

            if (isset($menulist[$i]) && $menulist[$i]['type'] == 'group') {
                $group_menu[$i] = $menulist[$i];
                for ($c=0; $c < count($menulist[$i]['items']); $c++) {
                    $group_menu[$i]['items'] = [];
                    //If user access is not allowed remove from array
                    if (isset($menulist[$i]['items'][$c]['id']) && in_array($menulist[$i]['items'][$c]['id'], $user_access)) {
                        if ($menulist[$i]['items'][$c]['route'] == $route) {
                            $menulist[$i]['items'][$c]['status'] = 'active';
                            $menulist[$i]['status'] = 'active';
                        }
                        $group_menu[$i]['items'][] = $menulist[$i]['items'][$c];
                        $new_menu[] = $group_menu[$i];
                    }

                }
            }
        }
        // \Log::info("NEW MENU", $new_menu);
        return $new_menu;
    }

    public static function notificationSetter()
    {
        $user = \Auth::user();
        $notifications = \App\Models\Broadcast::where('user_id', $user->id)->where('viewed', 0)->with('promo')->orderBy('created_at', 'DESC')->get();
        return $notifications;
    }

    public static function getUIDMd5($extra='', $length=8)
    {
        return substr(md5(uniqid(). microtime() . Str::random() . $extra), 0, $length);
    }

    public static function getUIDSha1($extra='', $length=8)
    {
        return substr(sha1(uniqid(). microtime() . Str::random() . $extra), 0, $length);
    }

    public static function getNumericUniqueId()
    {
        return str_pad(mt_rand(0, 999999999), 9, '0', STR_PAD_LEFT);
    }

    public static function saveToCache($id, $encodedData, $min)
    {
        if (!\Cache::has($id)) {
            \Cache::add($id, $encodedData, $min);
        } else {
            \Cache::put($id, $encodedData, $min);
        }
    }

    public static function convertNumberToWords($number) {

        $hyphen = '-';
        $conjunction = ' and ';
        $separator = ', ';
        $negative = 'negative ';
        $decimal = ' point ';
        $dictionary = array(
            0 => 'zero',
            1 => 'one',
            2 => 'two',
            3 => 'three',
            4 => 'four',
            5 => 'five',
            6 => 'six',
            7 => 'seven',
            8 => 'eight',
            9 => 'nine',
            10 => 'ten',
            11 => 'eleven',
            12 => 'twelve',
            13 => 'thirteen',
            14 => 'fourteen',
            15 => 'fifteen',
            16 => 'sixteen',
            17 => 'seventeen',
            18 => 'eighteen',
            19 => 'nineteen',
            20 => 'twenty',
            30 => 'thirty',
            40 => 'forty',
            50 => 'fifty',
            60 => 'sixty',
            70 => 'seventy',
            80 => 'eighty',
            90 => 'ninety',
            100 => 'hundred',
            1000 => 'thousand',
            1000000 => 'million',
            1000000000 => 'billion',
            1000000000000 => 'trillion',
            1000000000000000 => 'quadrillion',
            1000000000000000000 => 'quintillion'
        );

        if (!is_numeric($number)) {
            return false;
        }

        if (($number >= 0 && (int) $number < 0) || (int) $number < 0 - PHP_INT_MAX) {
            // overflow
            trigger_error(
                    'convert_number_to_words only accepts numbers between -' . PHP_INT_MAX . ' and ' . PHP_INT_MAX, E_USER_WARNING
            );
            return false;
        }
        if ($number < 0) {
            return $negative . convert_number_to_words(abs($number));
        }

        $string = $fraction = null;

        if (strpos($number, '.') !== false) {
            list($number, $fraction) = explode('.', $number);
        }

        switch (true) {
            case $number < 21:
                $string = $dictionary[$number];
                break;
            case $number < 100:
                $tens = ((int) ($number / 10)) * 10;
                $units = $number % 10;
                $string = $dictionary[$tens];
                if ($units) {
                    $string .= $hyphen . $dictionary[$units];
                }
                break;
            case $number < 1000:
                $hundreds = $number / 100;
                $remainder = $number % 100;
                $string = $dictionary[$hundreds] . ' ' . $dictionary[100];
                if ($remainder) {
                    $string .= $conjunction . convert_number_to_words($remainder);
                }
                break;
            default:
                $baseUnit = pow(1000, floor(log($number, 1000)));
                $numBaseUnits = (int) ($number / $baseUnit);
                $remainder = $number % $baseUnit;
                $string = convert_number_to_words($numBaseUnits) . ' ' . $dictionary[$baseUnit];
                if ($remainder) {
                    $string .= $remainder < 100 ? $conjunction : $separator;
                    $string .= convert_number_to_words($remainder);
                }
                break;
        }

        if (null !== $fraction && is_numeric($fraction)) {
            $string .= $decimal;
            $words = array();
            foreach (str_split((string) $fraction) as $number) {
                $words[] = $dictionary[$number];
            }
            $string .= implode(' ', $words);
        }

        return $string;
    }

    public static function shortMoneyFormat($number, $is_currency=true) {
        $precision = 2;
        if($number<100000) {
            if(!$is_currency) {
                $precision = 1;
            }
            return number_format($number, $precision, '.', ',');
        }
        if($number<1000000) {
            $precision = 1;
        }
        $units = ['', 'K', 'M', 'B', 'T', 'P', 'E', 'Z', 'Y'];

        $number = max($number, 0);
        $pow = floor(($number ? log($number) : 0) / log(1000));
        $pow = min($pow, count($units) - 1);
        $number /= pow(1000, $pow);

        return round($number, $precision) . $units[$pow];
    }

    /**
     * Format price
     * @param float $number
     * @return string
     */
    public static function priceFormat($number = 0, $thousands_sep='') {
        return is_numeric($number) ? number_format($number, 2, '.', $thousands_sep) : '0.00';
    }

    /**
     * Format price with currency sign: P20.03 or -P20.03
     * @param float $number
     * @return string
     */
    public static function currencyPriceFormat($number = 0, $currency = 'P', $space = true) {
        $number = $number>=0 ? $currency.($space ? " " : "").static::priceFormat($number, ',') : '-'.$currency.($space ? " " : "").static::priceFormat(abs($number), ',');
        return $number;
    }

    public static function displayDate($format='M d, Y h:i a', $datetime='') {
        return date($format, strtotime($datetime));
    }

    public static function properCase($words)
    {
        return ucwords(str_replace("_", " ", $words));
    }

    public static function formatDateTime($format='Y-m-d H:i:s', $datetime='')
    {
        try {
            return date($format, strtotime($datetime));
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

}
