<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\API\BaseController as BaseController;
use App\Models\Broadcast;
use App\Models\Notification;
use App\Models\User;
use Illuminate\Http\Request;
use App\Mail\BroadcastEmail;

class BroadcastController extends BaseController
{
    public function broadcastViaEmail(Request $request)
    {
        try {
            //Find all notification that are within the start and end date range and is an email format
            $date = date('Y-m-d');
            $notifications = Notification::where('format', 'email')
                ->whereDate('start', '<=', $date)
                ->whereDate('end','>=', $date)
                ->limit(2)
                ->get();
            if (! $notifications->isEmpty()) {
                foreach ($notifications->toArray() as $notif) {
                    $users = User::whereNull('deleted_at')->with('profile')->limit(2)->get();
                    foreach ($users as $user) {

                        $broadcastDone = Broadcast::where('user_id', $user->id)->where('notif_id', $notif['id'])->where('format', 'email')->first();
                        if (is_null($broadcastDone)) {
                            $person = !empty($user->profile->firstname) ? $user->profile->firstname : $user->email;
                            $email = $user->email;
                            $details = [
                                'title' => 'Check out '.env('APP_NAME').'!',
                                'appname' => env('APP_NAME'),
                                'person' => $person,
                                'email' => $email,
                                'promo' => $notif
                            ];
                            \Mail::to($email)->send(new BroadcastEmail($details));
                            $broadcast = new Broadcast();
                            $broadcast->user_id = $user->id;
                            $broadcast->notif_id = $notif['id'];
                            $broadcast->format = 'email';
                            $broadcast->save();
                        }

                    }
                }

            }

            return $this->sendResponse([], 'Email broadcasting is done.');

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function detail(Request $request, $id=0)
    {
        try {
            $user = $request->user();
            $broadcast = Broadcast::where('id', $id)->with('promo')->first();
            if (is_null($broadcast)) {
                return $this->sendError('Notification not found.', [], 406);
            }
            if ($broadcast->viewed < 1) {
                $broadcast->viewed = 1;
                $broadcast->save();
            }
            $userform = view('pages/promos/promo-details', [
                'promo'  => $broadcast->promo,
            ])->render();

            return $this->sendResponse(['user_form' => $userform], 'User form.');

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function readAll(Request $request)
    {
        try {
            $user = $request->user();
            $broadcast = Broadcast::where('user_id', $user->id)->where('viewed', 0)->get();
            if (!is_null($broadcast)) {
                foreach($broadcast as $bitem) {
                    $bitem->viewed = 1;
                    $bitem->save();
                }
            }

            return $this->sendResponse([], 'All notifications are marked as read.');

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
}
