<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\API\BaseController as BaseController;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;
use Illuminate\Http\Request;
use App\Models\Company;
use App\Helpers\Utilities;

class CompanyController extends BaseController
{

    public function readCompany(Request $request, $id=0)
    {
        try {
            $user = $request->user();
            $company = Company::where('id', $id)->where('user_id', $user->id)->first();
            if ($company) {
                return $this->sendResponse(['result' => $company], 'Here is the company details.');
            }
            return $this->sendError("This company does not exist.", [], 406);

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function create(Request $request)
    {
        try {
            $user = $request->user();
            $data = $request->all();
            $validator = Validator::make($data, ['name' => 'required|unique:company']);
            if ($validator->errors()->first()) {
                return $this->sendError('Check these fields first before completing the login request.', $validator->errors(), 406);
            }

            $company = Company::where('name', $data['name'])->first();
            if ($company) {
                return $this->sendError("The company name: `{$data['name']}` is already taken.", [], 406);
            }

            $extra = date('Y-m-d H:i:s').$user->id;
            $newCompany = new Company();
            $newCompany->name = $data['name'];
            $newCompany->user_id = $user->id;
            $newCompany->address = $data['address'];
            $newCompany->unique_key = (new Utilities)->generateHash($extra);
            $newCompany->save();
            return $this->sendResponse(['result' => $newCompany], 'Here is the new company details.');

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function edit(Request $request)
    {
        try {
            $user = $request->user();
            $data = $request->all();
            $company = Company::where('id', $data['id'])->where('user_id', $user->id)->first();
            if (!$company) {
                return $this->sendError("This company does not exist.", [], 406);
            }
            $validator = Validator::make($data, ['name' => 'required']);
            if ($validator->errors()->first()) {
                return $this->sendError('Check these fields first before completing the login request.', $validator->errors(), 406);
            }

            $company->name = $data['name'];
            $company->address = $data['address'];
            $company->save();
            return $this->sendResponse(['result' => $company], 'Company details has been updated.');

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function delete(Request $request)
    {
        try {
            $user = $request->user();
            $data = $request->all();
            $company = Company::where('id', $data['id'])->where('user_id', $user->id)->first();
            $company = Company::where('id', $data['id'])->where('user_id', $user->id)->first();
            if (!$company) {
                return $this->sendError("This company does not exist.", [], 406);
            }
            if ($company->agents->count() > 0) {
                return $this->sendError("This company has attached agents. Do you want to remove all attached items?", [], 406);
            }

            $company->delete();
            return $this->sendResponse([], 'Company has been deleted');

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function deleteAll(Request $request)
    {
        try {
            $user = $request->user();
            $data = $request->all();
            $company = Company::where('id', $data['id'])->where('user_id', $user->id)->first();
            if ($company->agents->count() > 0) {
                $company->agents->delete();
            }

            $company->delete();
            return $this->sendResponse([], 'Company has been deleted');

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
}
