<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\API\BaseController as BaseController;

use App\Helpers\MediaHelper;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

use App\Models\Media;

class MediaController extends BaseController
{
    public function item(Request $request)
    {
        return null;
    }

    public function upload(Request $request)
    {
        try {
            $data = $request->all();
            $mediaRule = (new MediaHelper)->returnMediaRule($data['type']);
            //NOTE: If application/octet-stream file is failing check upload limit on php.ini
            //First validation
            $validatorReqs = Validator::make($data, [
                'item_id' => 'required',
                'item_reference' => 'required|in:'.implode(',', config('uploads')['item_reference']),
                'type' => 'required|in:'.implode(',', config('uploads')['types']),
                'item' => $mediaRule
            ]);
            if ($validatorReqs->errors()->first()) {
                return $this->sendError($validatorReqs->errors(), [], 406);
            }

            $validator = Validator::make($data, ['item' => $mediaRule]);
            if ($validator->errors()->first()) {
                return $this->sendError($validatorReqs->errors(), [], 406);
            }

            $paths = (new MediaHelper)->storeItemToS3($data, $request);

            if (!is_null($paths['src']) && !is_null($paths['thumb']) ) {
                $media = new Media();
                $media->item_id = $data['item_id'];
                $media->item_id_reference = $data['item_reference'];
                $media->type = $data['type'];
                $media->caption = isset($data['caption']) ? $data['caption'] : '';
                $media->src = $paths['src'];
                $media->thumb = $paths['thumb'];
                $media->save();

                //Attach item data to update the front-end stored data
                $media = (new MediaHelper)->attachItemData($media);

                return $this->sendResponse($media, 'Item has been uploaded.');
            } else {
                \Log::info('Data payload:');
                // \Log::info((array) $data);
                return $this->sendError('Uploaded item failed.', [], 406);
            }
        } catch(\Exception $e) {
            \Log::error($e);
        }
    }

    public function delete(Request $request)
    {
        try {
            $data = $request->all();
            $user = $request->user();
            $validator = Validator::make($data, [
                'id' => 'required|exists:media,id,deleted_at,NULL',
                'item_id' => 'required',
                'item_reference' => 'required|in:'.implode(',', config('uploads')['item_reference']),
                'type' => 'required|in:'.implode(',', config('uploads')['types']),
            ]);

            if ($validator->fails()) {
                return $this->sendError('Check these fields first before completing this request.', $validator->errors(), 406);
            }

            $itemExist = Media::where('id', $data['id'])->where('item_id', $data['item_id'])
                        ->where('item_id_reference',$data['item_reference'])->where('type', $data['type'])->first();

            if (!is_null($itemExist)) {
                $deleteDone = (new MediaHelper)->deleteItemOnS3($itemExist);
                if ($deleteDone) {
                    $itemExist->delete();
                    return $this->sendResponse([], 'This item is deleted.');
                } else {
                    return $this->sendError('Something happened. We can not delete this item. Contact administrator.', [], 406);
                }
            } else {
                return $this->sendError('Item does not exist on record.', [], 406);
            }

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function itemImages(Request $request, $type='', $id=0)
    {
        try {
            $mediaExist = Media::where('id', $id)->where('type', $type)->first();
            if (!is_null($mediaExist)) {
                $path = \Storage::disk('s3')->temporaryUrl($mediaExist->src, now()->addMinute());
                return $this->sendResponse($path, 'Here is your media item.');
            }

            return $this->sendError('Media not found.', [], 406);
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
}
