<?php

namespace App\Http\Controllers\API;

use App\Helpers\MediaHelper;
use App\Http\Controllers\API\BaseController as BaseController;
use Illuminate\Support\Facades\Validator;
use Illuminate\Http\Request;
use App\Models\User;
use App\Models\Transaction;
use App\Models\Notification;
use App\Helpers\UserHelper;
use App\Helpers\ProfileHelper;
use App\Helpers\SecurityHelper;
use Chikolokoy08\Dengine\Dengine;

class NotificationController extends BaseController
{
    public function list(Request $request)
    {
        try {

            $data = $request->all();
            $user_id = $request->user()->id;

            $dengine = new Dengine();
            $dengine::parse($data);

            //MODEL INSTANCE
            $model = new Notification();
            $dtparams = $dengine::setParameters(['user_id' => $user_id]);
            //TOTAL RECORDS QUERY
            $total = count($model->getNotificationFeeds($dtparams));
            //QUERY WITH FULL PARAMETERS
            $query = $model->getNotificationFeeds($dtparams);
            //SETTER FOR TOTAL
            $dengine::recordsTotal($total);

            //PREPARING QUERY RESULT INTO DATATABLE JSON FORMAT
            $dengine::prepare($query);

            //RETURN AS JSON OBJECT
            return $dengine::make();

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function detail(Request $request, $id=0)
    {
        try {
            $user = $request->user();
            $transaction = Notification::where('id', $id)->first();
            if (is_null($transaction)) {
                return $this->sendError('Record not found.', [], 406);
            }
            return $this->sendResponse($transaction, 'Notification data.');

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function create(Request $request)
    {
        try {
            $data = $request->all();
            $user = $request->user();
            $validator = Validator::make($data, [
                'promo_name' => 'required',
                'promo_details' => 'required',
                'format' => 'required|in:email,platform',
                'start' => 'required',
                'end' => 'required',
                'broadcast' => 'required|in:yes,no'
            ]);

            if ($validator->fails()) {
                return $this->sendError('Check these fields first before completing this request.', $validator->errors()->first(), 406);
            }

            $notification = new Notification();
            if (isset($data['duplicate']) && !is_null($data['duplicate'])) {
                $name_count = Notification::where('promo_name', $data['promo_name'])->count();
                $data['promo_name'] = $data['promo_name'] .' '. ($name_count > 0 ? $name_count+1 : '' );
            }
            $notification->promo_name = trim($data['promo_name']);
            $notification->promo_details = $data['promo_details'];
            $notification->format = $data['format'];
            $notification->start = date('Y-m-d H:i:s', strtotime($data['start']));
            $notification->end = date('Y-m-d H:i:s', strtotime($data['end']));
            $notification->broadcast = $data['broadcast'] == 'yes';
            if (isset($data['points_qualifications']) && !is_null($data['points_qualifications'])) {
                $notification->points_qualifications = $data['points_qualifications'];
            }
            if (isset($data['rule']) && !is_null($data['rule'])) {
                $notification->rule = $data['rule'];
            }
            $notification->save();
            return $this->sendResponse(['notif' => $notification], 'Notification data has been created.');


        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function delete(Request $request)
    {
        try {
            $data = $request->all();
            $user = $request->user();
            $validator = Validator::make($data, ['notif_id' => 'required|exists:notifications,id']);

            if ($validator->fails()) {
                return $this->sendError('Check these fields first before completing this request.', $validator->errors()->first(), 406);
            }

            $notification = Notification::where('id', $data['notif_id'])->first();
            if (!is_null($notification)) {
                $notification->delete();
                return $this->sendResponse([], 'Notification data is successfully deleted.');
            } else {
                return $this->sendError('Notification data not found.', [], 406);
            }

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function uploadPoster(Request $request)
    {
        try {
            $data = $request->all();
            $user = $request->user();
            $validator = Validator::make($data, [
                'notif_id' => 'required|exists:notifications,id',
                'item' => 'required|mimetypes:image/jpeg,image/png,image/jpg,image/gif,image/svg,application/octet-stream',
            ]);

            if ($validator->fails()) {
                return $this->sendError('Check these fields first before completing this request.', $validator->errors(), 406);
            }

            if ($user->role != 'admin' && $user->id != $data['user_id']) {
                return $this->sendError('Update request is not allowed.', $validator->errors(), 406);
            }

            $data['type'] = 'images';
            $paths = (new MediaHelper)->storeItem('local', $data, $request);

            if (!is_null($paths['src']) && !is_null($paths['thumb']) ) {
                $notif = Notification::where('id', $data['notif_id'])->first();
                $notif->poster = asset('storage/'.$paths['src']);
                $notif->save();
                return $this->sendResponse(['notif' => $notif], 'Item has been uploaded.');
            } else {
                return $this->sendError('Uploaded item failed.', [], 406);
            }

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
}
