<?php

namespace App\Http\Controllers\API;

use App\Helpers\MediaHelper;
use App\Http\Controllers\API\BaseController as BaseController;
use Illuminate\Support\Facades\Validator;
use Illuminate\Http\Request;
use App\Models\User;
use App\Models\Transaction;
use App\Models\Promo;
use App\Helpers\UserHelper;
use App\Models\ClaimedPromo;
use App\Helpers\ProfileHelper;
use App\Helpers\SecurityHelper;
use Chikolokoy08\Dengine\Dengine;

class PromoController extends BaseController
{
    public function list(Request $request)
    {
        try {

            $data = $request->all();
            $user_id = $request->user()->id;

            $dengine = new Dengine();
            $dengine::parse($data);

            //MODEL INSTANCE
            $model = new Promo();
            $dtparams = $dengine::setParameters(['user_id' => $user_id]);
            //TOTAL RECORDS QUERY
            $total = count($model->getPromoFeeds($dtparams));
            //QUERY WITH FULL PARAMETERS
            $query = $model->getPromoFeeds($dtparams);
            //SETTER FOR TOTAL
            $dengine::recordsTotal($total);

            //PREPARING QUERY RESULT INTO DATATABLE JSON FORMAT
            $dengine::prepare($query);

            //RETURN AS JSON OBJECT
            return $dengine::make();

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function detail(Request $request, $id=0)
    {
        try {
            $user = $request->user();
            $promo = Promo::where('id', $id)->first();
            if (is_null($promo)) {
                return $this->sendError('Record not found.', [], 406);
            }
            return $this->sendResponse($promo, 'Promo data.');

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function create(Request $request)
    {
        try {
            $data = $request->all();
            $user = $request->user();
            $validator = Validator::make($data, [
                'promo_name' => 'required',
                'promo_details' => 'required',
                'start' => 'required',
                'end' => 'required|after:start',
                'broadcast' => 'required|in:yes,no'
            ]);

            if ($validator->fails()) {
                return $this->sendError('Check these fields first before completing this request.', $validator->errors()->first(), 406);
            }

            $promo = new Promo();
            $name_count = Promo::where('promo_name', $data['promo_name'])->count();
            $data['promo_name'] = $data['promo_name'] .' '. ($name_count > 0 ? $name_count+1 : '' );

            $promo->promo_name = trim($data['promo_name']);
            $promo->promo_details = $data['promo_details'];
            $promo->start = date('Y-m-d H:i:s', strtotime($data['start']));
            $promo->end = date('Y-m-d H:i:s', strtotime($data['end']));
            $promo->broadcast = $data['broadcast'] == 'yes';
            if (isset($data['point_qualifications']) && !is_null($data['point_qualifications'])) {
                $promo->point_qualifications = $data['point_qualifications'];
            }
            if (isset($data['rule']) && !is_null($data['rule'])) {
                $promo->rule = $data['rule'];
            }

            $promo->save();
            return $this->sendResponse(['promo' => $promo], 'Promo data has been created.');


        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function update(Request $request)
    {
        try {
            $data = $request->all();
            $user = $request->user();
            $validator = Validator::make($data, [
                'promo_id' => 'required|exists:promotions,id',
                'promo_name' => 'required',
                'promo_details' => 'required',
                'start' => 'required',
                'end' => 'required|after:start',
                'broadcast' => 'required|in:yes,no'
            ]);

            if ($validator->fails()) {
                return $this->sendError('Check these fields first before completing this request.', $validator->errors()->first(), 406);
            }

            $promo = Promo::where('id', $data['promo_id'])->first();
            $name_count = Promo::where('promo_name', $data['promo_name'])->where('id', '!=', $data['promo_id'])->count();
            $data['promo_name'] = $data['promo_name'] .' '. ($name_count > 0 ? $name_count+1 : '' );

            $promo->promo_name = trim($data['promo_name']);
            $promo->promo_details = $data['promo_details'];
            $promo->start = date('Y-m-d H:i:s', strtotime($data['start']));
            $promo->end = date('Y-m-d H:i:s', strtotime($data['end']));
            $promo->broadcast = $data['broadcast'] == 'yes';
            if (isset($data['point_qualifications']) && !is_null($data['point_qualifications'])) {
                $promo->point_qualifications = $data['point_qualifications'];
            }
            if (isset($data['rule']) && !is_null($data['rule'])) {
                $promo->rule = $data['rule'];
            }

            $promo->save();
            return $this->sendResponse(['promo' => $promo], 'Promo data has been updated.');


        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function delete(Request $request)
    {
        try {
            $data = $request->all();
            $user = $request->user();
            $validator = Validator::make($data, ['promo_id' => 'required|exists:promotions,id']);

            if ($validator->fails()) {
                return $this->sendError('Check these fields first before completing this request.', $validator->errors()->first(), 406);
            }

            $promo = Promo::where('id', $data['promo_id'])->first();
            if (!is_null($promo)) {
                $promo->delete();
                return $this->sendResponse([], 'Promo data is successfully deleted.');
            } else {
                return $this->sendError('Promo data not found.', [], 406);
            }

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function uploadPoster(Request $request)
    {
        try {
            $data = $request->all();
            $user = $request->user();
            $validator = Validator::make($data, [
                'promo_id' => 'required|exists:promotions,id',
                'item' => 'required|mimetypes:image/jpeg,image/png,image/jpg,image/gif,image/svg,application/octet-stream',
            ]);

            if ($validator->fails()) {
                return $this->sendError('Check these fields first before completing this request.', $validator->errors(), 406);
            }

            if ($user->role != 'admin' && $user->id != $data['user_id']) {
                return $this->sendError('Update request is not allowed.', $validator->errors(), 406);
            }

            $data['type'] = 'images';
            $paths = (new MediaHelper)->storeItem('local', $data, $request);

            if (!is_null($paths['src']) && !is_null($paths['thumb']) ) {
                $promo = Promo::where('id', $data['promo_id'])->first();
                $promo->poster = asset('storage/'.$paths['src']);
                $promo->save();
                return $this->sendResponse(['promo' => $promo], 'Item has been uploaded.');
            } else {
                return $this->sendError('Uploaded item failed.', [], 406);
            }

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function promoForm(Request $request, $pid=0)
    {
        try {
            $data = $request->all();
            $user = \Auth::user();
            if ($user->role == 'admin') {
                $mode = 'create';
                $promo = [];
                $qrcode = null;
                if ($pid > 0) {
                    $mode = 'edit';
                    $promo = Promo::where('id', $pid)->first();
                }
                $userform = view('pages/promos/promo-create-edit', [
                    'mode'  => $mode,
                    'promo'  => $promo,
                ])->render();

                return $this->sendResponse(['user_form' => $userform], 'User form.');

            } else {
                return $this->sendError('This request is not allowed.', [], 406);
            }
        } catch (\Exception $e) {
            \Log::error($e->getMessage());
            \Log::error($e);
        }
    }

    public function promoClaimForm(Request $request, $pid=0)
    {
        try {
            $data = $request->all();
            $user = \Auth::user();
            if ($user->role == 'admin') {
                $mode = 'create';
                $promo = [];
                if ($pid > 0) {
                    $mode = 'edit';
                    $promo = Promo::where('id', $pid)->first();
                }
                $userform = view('pages/promos/promo-claim', [
                    'mode'  => $mode,
                    'promo'  => $promo,
                ])->render();

                return $this->sendResponse(['user_form' => $userform], 'User form.');

            } else {
                return $this->sendError('This request is not allowed.', [], 406);
            }
        } catch (\Exception $e) {
            \Log::error($e->getMessage());
            \Log::error($e);
        }
    }

    public function startClaim(Request $request)
    {
        try {
            $data = $request->all();
            $user = $request->user();
            $validator = Validator::make($data, [
                'promo_id'      => 'required|exists:promotions,id',
                'customer_id'   => 'required|exists:users,id'
            ]);

            if ($validator->fails()) {
                return $this->sendError('Check these fields first before completing this request.', $validator->errors()->first(), 406);
            }

            //Check if promo requires points
            $promo = Promo::where('id', $data['promo_id'])->first();
            $dateNow = date('Y-m-d H:i:s');
            //Check if promo is still valid
            if ($dateNow >= $promo->start && $dateNow <= $promo->end) {
                $targetPoints = number_format($promo->point_qualifications);
                $points = UserHelper::getTotalPoints($data['customer_id']);
                if ($targetPoints > 0) {

                    if ($points < $targetPoints) {
                        //Check if acquired points exceeds or equal the target points qualification
                        return $this->sendError('Check these fields first before completing this request.', 'Promo requires '.$targetPoints.' points before the customer can claim it. Customer only has '.$points.' points', 406);
                    } else {
                        //Claim points from transactions
                        $transactions = Transaction::where('user_id', $data['customer_id'])->where('claimed', 0)->get();
                        if (!is_null($transactions)) {
                            $pointsClaimed = 0;
                            foreach($transactions as $titem) {
                                if ($pointsClaimed != $targetPoints) {
                                    $pointsClaimed = $pointsClaimed+$titem->points;
                                    $titem->claimed = 1;
                                    if ($pointsClaimed > $targetPoints) {
                                        $lessThis = $pointsClaimed-$targetPoints;
                                        $pointsClaimed = $pointsClaimed-$lessThis;
                                        $titem->points = $lessThis;
                                        $titem->claimed = 0;
                                    }
                                    $points = $pointsClaimed;
                                    $titem->save();
                                }
                            }
                        } else {
                            return $this->sendError('Check these fields first before completing this request.', 'Customer should have at least 1 point to claim any promo.', 406);
                        }
                    }

                }

                $claimed = new ClaimedPromo();
                $claimed->user_id = $data['customer_id'];
                $claimed->promo_id = $data['promo_id'];
                $claimed->cashier_id = $user->id;
                $claimed->points_claimed = $points;
                $claimed->save();

                return $this->sendResponse([], 'Promo successfully claimed.');


            } else {
                return $this->sendError('Check these fields first before completing this request.', 'This promo is expired. Check start and end dates.', 406);
            }




        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
}
