<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\API\BaseController as BaseController;
use Illuminate\Support\Facades\Validator;
use Illuminate\Http\Request;
use App\Models\Transaction;
use App\Models\Service;
use Chikolokoy08\Dengine\Dengine;
use App\Helpers\Utilities;

class ServiceController extends BaseController
{
    public function list(Request $request)
    {
        try {

            $data = $request->all();
            $user = $request->user();
            $user_id = $user->id;

            $dengine = new Dengine();
            $dengine::parse($data);

            //MODEL INSTANCE
            $model = new Service();
            $dtparams = $dengine::setParameters(['user_id' => $user_id, 'role' => $user->role]);
            //TOTAL RECORDS QUERY
            $total = count($model->getServicesFeeds($dtparams));
            //QUERY WITH FULL PARAMETERS
            $query = $model->getServicesFeeds($dtparams);
            //SETTER FOR TOTAL
            $dengine::recordsTotal($total);

            //PREPARING QUERY RESULT INTO DATATABLE JSON FORMAT
            $dengine::prepare($query);

            //RETURN AS JSON OBJECT
            return $dengine::make();

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function detail(Request $request, $id=0)
    {
        try {
            $user = $request->user();
            $transaction = Transaction::where('id', $id)->with('user','cashier','profile','cashprof')->first();
            if (is_null($transaction)) {
                return $this->sendError('Record not found.', [], 406);
            }
            return $this->sendResponse($transaction, 'Transaction data.');

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function create(Request $request)
    {
        try {
            $data = $request->all();
            $user = $request->user();
            $validator = Validator::make($data, [
                'name' => 'required',
                'cost' => 'required|numeric|between:-9999999999.99,9999999999.99',
                'points' => 'required|digits_between:1,99999999999999',
                'status' => 'required|in:available,unavailable'
            ]);

            if ($validator->fails()) {
                return $this->sendError('Check these fields first before completing this request.', $validator->errors()->first(), 406);
            }

            $service = new Service();
            $service->user_id = $user->id;
            $service->name = $data['name'];
            $service->cost = $data['cost'];
            $service->points = $data['points'];
            $service->status = $data['status'] == 'available';
            $service->save();
            return $this->sendResponse(['service' => $service], 'Service data has been created.');


        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function update(Request $request)
    {
        try {
            $data = $request->all();
            $user = $request->user();
            $validator = Validator::make($data, [
                'service_id' => 'required|exists:services,id',
                'name' => 'required',
                'cost' => 'required|numeric|between:-9999999999.99,9999999999.99',
                'points' => 'required|digits_between:1,99999999999999',
                'status' => 'required|in:available,unavailable'
            ]);

            if ($validator->fails()) {
                return $this->sendError('Check these fields first before completing this request.', $validator->errors()->first(), 406);
            }

            $service = Service::where('id', $data['service_id'])->first();
            $service->name = $data['name'];
            $service->cost = $data['cost'];
            $service->points = $data['points'];
            $service->status = $data['status'] == 'available';
            $service->save();
            return $this->sendResponse(['service' => $service], 'Service data has been updated.');

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function delete(Request $request)
    {
        try {
            $data = $request->all();
            $user = $request->user();
            $validator = Validator::make($data, ['service_id' => 'required|exists:services,id']);

            if ($validator->fails()) {
                return $this->sendError('Check these fields first before completing this request.', $validator->errors()->first(), 406);
            }

            $service = Service::where('id', $data['service_id'])->first();
            if (!is_null($service)) {
                $service->delete();
                return $this->sendResponse([], 'Servuce data is successfully deleted.');
            } else {
                return $this->sendError('Service data not found.', [], 406);
            }

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function servicesForm(Request $request, $sid=0)
    {
        try {
            $data = $request->all();
            $user = \Auth::user();
            if ($user->role == 'admin') {
                $mode = 'create';
                $service = [];
                if ($sid > 0) {
                    $mode = 'edit';
                    $service = Service::where('id', $sid)->first();
                }
                $userform = view('pages/services/service-create-edit', [
                    'mode'  => $mode,
                    'service'  => $service,
                ])->render();

                return $this->sendResponse(['user_form' => $userform], 'User form.');

            } else {
                return $this->sendError('This request is not allowed.', [], 406);
            }
        } catch (\Exception $e) {
            \Log::error($e->getMessage());
            \Log::error($e);
        }
    }
}
