<?php

namespace App\Http\Controllers\API;

use App\Helpers\MediaHelper;
use App\Http\Controllers\API\BaseController as BaseController;
use Illuminate\Support\Facades\Validator;
use Illuminate\Http\Request;
use App\Models\User;
use App\Models\UserProfile;
use App\Models\Transaction;
use App\Helpers\UserHelper;

class UserController extends BaseController
{

    public function currentUser(Request $request)
    {
        try {
            $user = $request->user();
            return User::where('id', $user->id)->with('profile')->first();
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function fullProfile(Request $request, $user_id=0)
    {
        try {
            return User::where('id', $user_id)->with('profile')->withCount('followers','followings')->first();
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function userProfile(Request $request, $user_id=0)
    {
        try {
            $user = UserProfile::where('id', $user_id)->withCount('followers','followings')->first();
            if ($user) {
                return $this->sendResponse(['result' => $user], 'Here is the user profile.');
            } else {
                return $this->sendError('User profile not found.', [], 406);
            }
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function userCompany(Request $request)
    {
        try {
            $user = $request->user();
            return \App\Models\Company::where('user_id', $user->id)->get();
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function profileUpdate(Request $request)
    {
        try {
            $data = $request->all();
            $user = $request->user();
            $validator = Validator::make($data, [
                'email' => 'required|exists:users,email',
                'firstname' => 'required',
                'lastname' => 'required',
                'newsletter_validation' => 'required'
            ]);
            if (isset($data['password']) && !empty($data['password'])) {
                $rules['password'] = 'required|confirmed';
                $rules['password_confirmation'] = 'required';
            }

            if ($validator->fails()) {
                return $this->sendError('Check these fields first before completing this request.', $validator->errors()->first(), 406);
            }

            $profile = UserProfile::where('user_id', $user->id)->first();
            $profile->firstname = $data['firstname'];
            $profile->lastname = $data['firstname'];
            if (isset($data['password']) && !empty($data['password'])) {
                $user->password = $data['password'];
                $user->save();
            }

            if (isset($data['mobile']) && !empty($data['mobile'])) {
                $user->profile->mobile = trim($data['mobile']);
            }

            if (isset($data['address']) && !empty($data['address'])) {
                $user->profile->address = trim($data['address']);
            }
            $user->profile->save();
            $updatedUser = User::where('id', $user->id)->with('profile')->first();
            return $this->sendResponse(['user' => $updatedUser], 'User data has been updated.');

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function userUpdate(Request $request)
    {
        try {
            $data = $request->all();
            $user = $request->user();
            $validator = Validator::make($data, [
                'user_id' => 'required|exists:users,id',
                'firstname' => 'required',
                'lastname' => 'required',
                'newsletter_validation' => 'required'
            ]);

            if ($validator->fails()) {
                return $this->sendError('Check these fields first before completing this request.', $validator->errors(), 406);
            }

            if ($user->role != 'admin' && $user->id != $data['user_id']) {
                return $this->sendError('Update request is not allowed.', $validator->errors(), 406);
            }

            $profile = UserProfile::where('user_id', $data['user_id'])->first();
            $profile->firstname = $data['firstname'];
            $profile->lastname = $data['firstname'];
            if (isset($data['display_name'])) {
                $profile->display_name = $data['display_name'];
            }
            if (isset($data['mobile'])) {
                $profile->mobile = $data['mobile'];
            }
            $profile->save();
            $updatedUser = User::where('id', $user->id)->with('profile')->first();
            return $this->sendResponse(['user' => $updatedUser], 'User data has been updated.');

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function uploadAvatar(Request $request)
    {
        try {
            $data = $request->all();
            $user = $request->user();
            $validator = Validator::make($data, [
                'item' => 'required|mimetypes:image/jpeg,image/png,image/jpg,image/gif,image/svg,application/octet-stream',
            ]);

            if ($validator->fails()) {
                return $this->sendError('Check these fields first before completing this request.', $validator->errors()->first(), 406);
            }

            // if ($user->role != 'admin' && $user->id != $data['user_id']) {
            //     return $this->sendError('Update request is not allowed.', $validator->errors(), 406);
            // }

            $data['type'] = 'images';
            $paths = (new MediaHelper)->storeItem('local', $data, $request);

            if (!is_null($paths['src']) && !is_null($paths['thumb']) ) {
                $profile = UserProfile::where('user_id', $user->id)->first();
                $profile->avatar = asset('storage/'.$paths['src']);
                $profile->save();
                return $this->sendResponse(['profile' => $profile], 'Item has been uploaded.');
            } else {
                return $this->sendError('Uploaded item failed.', [], 406);
            }

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function updateStatus(Request $request)
    {
        try {
            $data = $request->all();
            $user = $request->user();
            $validator = Validator::make($data, ['user_id' => 'required|exists:users,id', 'status' => 'required|in:active,inactive']);

            if ($validator->fails()) {
                return $this->sendError('Check these fields first before completing this request.', $validator->errors(), 406);
            }

            if ($user->role != 'admin' && $user->id != $data['user_id']) {
                return $this->sendError('Update request is not allowed.', $validator->errors(), 406);
            }

            $user->status = $data['status'] == 'active';
            $user->save();
            return $this->sendResponse([], 'User status has been updated.');

        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function userForm(Request $request, $uid=0)
    {
        try {
            $data = $request->all();
            $user = \Auth::user();
            if ($user->role == 'admin') {
                $mode = 'create';
                $currentuser = [];
                $qrcode = null;
                $points = 0;
                if ($uid > 0) {
                    $mode = 'edit';
                    $currentuser = User::where('id', $uid)->with('profile')->first();
                    $points = UserHelper::getTotalPoints($uid);
                    $qrcode = UserHelper::generateQrCode($currentuser->profile->qrstring, 180);
                }
                $roles = $data['type'] == 'cashier' ? ['admin','cashier'] : ['customer'];
                $userform = view('pages/users/create-edit', [
                    'mode'  => $mode,
                    'user'  => $currentuser,
                    'roles' => $roles,
                    'qrcode'=> $qrcode,
                    'points' => $points
                ])->render();

                return $this->sendResponse(['user_form' => $userform], 'User form.');

            } else {
                return $this->sendError('This request is not allowed.', [], 406);
            }
        } catch (\Exception $e) {
            \Log::error($e->getMessage());
            \Log::error($e);
        }
    }

    public function userClaimForm(Request $request, $uid=0)
    {
        try {
            $data = $request->all();
            $user = \Auth::user();
            if ($user->role == 'admin') {
                $mode = 'create';
                $currentuser = [];
                $qrcode = null;
                $points = 0;
                if ($uid > 0) {
                    $mode = 'edit';
                    $currentuser = User::where('id', $uid)->with('profile')->first();
                    $points = UserHelper::getTotalPoints($uid);
                    $qrcode = UserHelper::generateQrCode($currentuser->profile->qrstring, 180);
                }
                $userform = view('pages/promos/promo-user-fields', [
                    'customer'  => $currentuser,
                    'qrcode'=> $qrcode,
                    'points' => $points
                ])->render();

                return $this->sendResponse(['user_form' => $userform], 'User form.');

            } else {
                return $this->sendError('This request is not allowed.', [], 406);
            }
        } catch (\Exception $e) {
            \Log::error($e->getMessage());
            \Log::error($e);
        }
    }
}
