<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class Transaction extends Model
{
    use HasFactory, SoftDeletes;

    protected $table = 'transactions';

    public function user()
    {
        return $this->hasOne(User::class,'id','user_id');
    }

    public function profile()
    {
        return $this->hasOne(User::class,'id','user_id');
    }

    public function cashier()
    {
        return $this->hasOne(User::class,'id','cashier_id');
    }

    public function cashprof()
    {
        return $this->hasOne(User::class,'id','cashier_id');
    }

    protected function txnDataQuery($whereClause='')
    {
        return "
            SELECT
                t.id,
                t.service,
                FORMAT(IFNULL(t.points, 0),0) as points_earned,
                u.id as customer_user_id,
                u.email as customer_email,
                CONCAT(up.firstname, ' ', up.lastname) as customer_name,
                cu.id as cashier_user_id,
                cu.email as cashier_email,
                CONCAT(cp.firstname, ' ', cp.lastname) as cashier_name,
                t.created_at
            FROM
                {$this->table} as t
                LEFT JOIN users as u ON u.id = t.user_id
                LEFT JOIN user_profiles as up ON up.user_id = t.user_id
                LEFT JOIN users as cu ON cu.id = t.cashier_id
                LEFT JOIN user_profiles as cp ON cp.user_id = t.cashier_id
            WHERE
                {$whereClause}
        ";
    }

    public function getTransactionFeeds($qp=[])
    {
        $column_orderable 	= [ 'id', 'service', 'customer_name', 'points_earned','cashier_name','created_at'];
		$qp['column_text']	= $qp['column'] == '' ? $column_orderable[0] : $column_orderable[$qp['column']];
		$qp['take']         = $qp['get_total'] == true || $qp['keyword'] != '' ? 5000000 : $qp['take'];
		$whereClause        = "ISNULL(t.deleted_at)";

        if (isset($qp['role']) && $qp['role'] == 'customer') {
            $whereClause .= " AND t.user_id = {$qp['user_id']}";
        }

		//IF KEYWORD IS NOT EMPTY
		if (!empty($qp['keyword'])) {
            $keyword = $qp['keyword'];
			$whereClause .= " AND (
                u.email LIKE '%$keyword%' OR up.firstname LIKE '%$keyword%' OR up.lastname LIKE '%$keyword%' OR
                cu.email LIKE '%$keyword%' OR cp.firstname LIKE '%$keyword%' OR cp.lastname LIKE '%$keyword%'
            )";
		}

        $query = $this->txnDataQuery($whereClause);
        $query .= "
            ORDER BY {$qp['column_text']} {$qp['order_type']}
		    LIMIT {$qp['skip']}, {$qp['take']}
        ";

        return \DB::select($query);

    }
}
