<?php

namespace Database\Factories;

use Illuminate\Database\Eloquent\Factories\Factory;
use Illuminate\Database\Eloquent\Factories\Sequence;
use Illuminate\Support\Str;

class UserFactory extends Factory
{
    /**
     * Define the model's default state.
     *
     * @return array
     */
    public function definition()
    {
        $email = $this->faker->unique()->safeEmail();
        $exp_username = explode('@',$email);
        $username = trim($exp_username[0]);
        return [
            'username' => $username,
            'email' => $email,
            'password' => 'P@s5w0d4dm13', // password
            'role' => 'mock_user',
            'remember_token' => Str::random(10),
            'last_login' => now(),
            'email_verified_at' => now(),
            'verified_at' => now()
        ];
    }

    public function demouser()
    {
        return $this->state(function (array $attributes) {
            return [
                'username' => 'demo',
                'email' => 'demo@mail.com',
                'password' => 'P@s5w0d4dm13', // password
                'role' => 'user',
                'remember_token' => Str::random(10),
                'status' => true,
                'last_login' => now(),
                'email_verified_at' => now(),
                'verified_at' => now()
            ];
        });
    }

    public function adminuser()
    {
        return $this->state(function (array $attributes) {
            return [
                'username' => 'admin',
                'email' => 'admin@mail.com',
                'password' => 'P@s5w0d4dm13', // password
                'role' => 'admin',
                'remember_token' => Str::random(10),
                'status' => true,
                'last_login' => now(),
                'email_verified_at' => now(),
                'verified_at' => now()
            ];
        });
    }

    public function configure()
    {
        return $this->afterMaking(function (\App\Models\User $user) {

        })->afterCreating(function (\App\Models\User $user) {

            $exp_username = explode('@',$user->email);
            $atname = trim($exp_username[0]);
            $setting = rand(0,1) == 1 ? 'public' : 'private';
            $preference = rand(0,1) == 1 ? 'no_ads' : 'with_ads';

            \App\Models\UserProfile::factory()->count(1)->state([
                'user_id' => $user->id,
                'firstname' => $this->faker->firstName(),
                'lastname' => $this->faker->lastName(),
                'display_name' => $atname,
                'at_name' => $atname,
                'setting' => $setting,
                'ad_preference' => $preference,
            ])->create();

            \App\Models\Company::factory()->count(1)->state([
                'user_id' => $user->id
            ])->create();
        });
    }
}
