"use strict";
(function(){
	var mod = {
		vars: {
			date_input: $(document).find('[name="bet-date"]'),
			bet_table_list: $(document).find('#bet-table-list'),
			print_holder: $(document).find('#print-holder'),
			utils: null,
			date_set: null,
			dt_btn: {
				file: $(document).find('.btn-export-report'),
				print: $(document).find('.btn-print-report'),
			},
			ticket_modal_el: $(document).find('#print-ticket-modal'),
			btn_cancel: $(document).find('.btn-cancel-ticket'),
			print_bind: false,
			user_role: 'cashier',
			ishistory: null
		},
		objs: {
			btl_obj: null,
			ticket_modal_obj: null,
			ticket_data: null
		},
		init: function() {
			mod.vars.utils = window.bsutils;
			mod.vars.ishistory = typeof ishistory != 'undefined' ? ishistory : false;
			mod.list.init();
			mod.buttons.init();
		},
		list: {
			init: function() {
				mod.list.date.init();
			},
			date: {
				init: function() {
					mod.vars.date_input.on('change', mod.list.date.onchange);
					mod.list.table.refresh();
					// mod.list.date.nowdate();
				},
				nowdate: function() {
					let now = new Date('2022-10-03');
					let date = now.toLocaleDateString("en-US",{year:"numeric",month:"2-digit",day:"2-digit"});
					let parseDate = date.split('/');
					let formattedDate = parseDate[2]+'-'+parseDate[0]+'-'+parseDate[1];
					mod.vars.date_set = formattedDate;
					mod.vars.date_input.val(formattedDate);
					mod.list.table.refresh();
				},
				onchange: function() {
					console.log($(this).val());
					mod.vars.date_set = $(this).val();
					mod.list.table.refresh();
				},
				latest: function() {
					console.log(mod.vars.date_set);
					return mod.vars.date_set;
				}
			},
			table: {
				refresh: function() {
					if (mod.objs.btl_obj == null) {
						mod.list.table.load();
					} else {
						console.log(mod.objs.btl_obj.ajax);
						mod.objs.btl_obj.ajax.reload();
					}
				},
				load: function() {
					let server = window.location.origin + (mod.vars.ishistory == true ? '/datatable/bet-list/history' : '/datatable/bet-list');
					console.log(server);
					mod.objs.btl_obj = mod.vars.bet_table_list.DataTable({
						// searching: false,
						paging: true,
						destroy: true,
						processing: true,
	        			serverSide: true,
	        			ajax: {
	                        url: server,
	                        data: function(d) {
	                        	d.ishistory = mod.vars.ishistory;
	                        	d._token = mod.vars.utils.vars.csrf;
	                        	d.date = mod.list.date.latest();
	                        },
	                        type: 'POST',
	                    },
	        			order: [[0, 'desc']],
	                    columnDefs: [{ "targets": "_all", "defaultContent": "" }],
	                    columns: [
	                    	{ "data":"id", "title":"ID", "orderable":true, "searchable":true },
	                    	{ "data":"cashier", "title":"Cashier", "orderable":true, "searchable":true },
	                    	{ "data":"fight_number", "title":"Fight #", "orderable":true, "searchable":true },
	                    	{ "data":"bet_key", "title":"Key", "orderable":true, "searchable":true },
	                    	{ 
	                    		"data":"bet_type", "title":"Type", "orderable":true, "searchable":true,
	                    		render: function(data, type, row) {
									return '<button href="#" class="btn btn-sm btn-'+data+' bet-type">'+mod.vars.utils.toTitleCase(data)+'</button>';
	                    		}
	                    	},
	                    	{ "data":"amount", "title":"Amount", "orderable":true, "searchable":true },
	                    	{ "data":"winnings", "title":"Payout", "orderable":true, "searchable":true },
	                    	{ 
	                    		"data":"status", "title":"Status", "orderable":true, "searchable":true,
	                    		render: function(data, type, row) {
									return mod.vars.utils.toTitleCase(data);
	                    		}
	                    	},
	                    	{ "data":"claim_payout", "title":"Claim Date", "orderable":true, "searchable":true },
	                    	{ "data":"created_at", "title":"Date", "orderable":true, "searchable":true },
	                    	{ 
	                    		"data":"unique_key", "title":"Actions", "orderable":false, "searchable":false,
	                    		render: function(data, type, row) {
									return '<button href="#" class="btn btn-sm btn-primary view-bet" data-bkey="'+data+'"><i class="fa fa-edit"></i></button>';
	                    		}
	                    	}
	                    ],
	                    initComplete: function(distable, data){
	                    },
	                    drawCallback: function() {
	                    	mod.ticket.view.init();
	                    	// console.log(mod.objs.btl_obj);
	                    	// console.log(mod.objs.btl_obj.data());
	                    	// console.log(mod.objs.btl_obj.ajax.params())
	                    }
					});	
				}
			},
		},
		buttons: {
			init: function() {
				mod.vars.dt_btn.file.on('click', mod.buttons.export.file.request);
				mod.vars.dt_btn.print.on('click', mod.buttons.print);
				mod.vars.btn_cancel.on('click', mod.ticket.cancel.init);
			},
			export: {
				file: {
					request: function(e) {
						e.preventDefault();
						if (mod.objs.btl_obj.data().length > 0) {
							let dtparams = mod.objs.btl_obj.ajax.params();
							dtparams.export_type = $(this).data('type');
							console.log(dtparams);
							mod.vars.utils.ajaxcall('POST', '/downloads/bets/request', dtparams).then(function(res){
								if (res.success) {
									if (res.data.new_window) {
										window.open(res.data.new_window);
									}
								}
							}, function(err) {
								console.log(err);
							});					
						} else {
							mod.vars.utils.prompt('error', 'Sorry! There is nothing to export on this current table.', 'Export table to PDF');
						}
					},
					download: function(response) {
						var blob = new Blob([response], {type: 'application/pdf'})
						var url = URL.createObjectURL(blob);
						location.assign(url);
					} 
				}
			},
			print: function(e) {
				e.preventDefault();
				if (mod.objs.btl_obj.data().length > 0) {
					mod.vars.print_holder.html('');
					let dtparams = mod.objs.btl_obj.ajax.params();
					mod.vars.utils.ajaxcall('POST', '/renders/bet/list/table', dtparams).then(function(res){
						if (res.success) {
							if (res.data.printable) {
								mod.vars.print_holder.append(res.data.printable);
								let printable = $(document).find('#printable-table');
								mod.vars.utils.print(printable, res.data.title, function() {
									mod.vars.print_holder.html('');
								});
							}
						}
					}, function(err) {
						console.log(err);
						mod.vars.print_holder.html('');
					});					
				} else {
					mod.vars.utils.prompt('error', 'Sorry! There is nothing to print on this current table.', 'Print table');
				}
			}
		},
		ticket: {
			cancel: {
				init: function(e) {
					e.preventDefault();
					let fight = mod.objs.ticket_data.fight;
					let bet = mod.objs.ticket_data.bet_info;
					if (confirm("Are you sure you want to cancel Bet#"+bet.unique_key+" of Fight#"+fight.id+"("+fight.unique_key+")?") == true) {
						mod.ticket.cancel.start();
					}
				},
				prep: function() {
					let fight = mod.objs.ticket_data.fight;
					let bet = mod.objs.ticket_data.bet_info;
					console.log(mod.vars.user_role);
					if (bet.status == 'cancelled' || fight.result != 'ongoing' || fight.bet_status == 'closed' || mod.vars.user_role == 'cashier') {
						mod.vars.btn_cancel.hide();
					} else {
						mod.vars.btn_cancel.removeClass('fade').show();
					}
				},
				start: function() {
					
					let params = {};
					let bkey = mod.objs.ticket_data.bet_info.unique_key;
					params['_token'] = mod.vars.utils.vars.csrf;
					params['unique-key'] = bkey;
					mod.vars.utils.ajaxcall('POST', '/bet/ticket/cancel', params).then(function(res) {
						console.log(res);
						if (res.success) {
							let modeType = 'Cancelled';
							mod.vars.utils.prompt('success', res.message, modeType+' Ticket');
							if (res.data.ticket) {
								mod.objs.ticket_data = res.data;
								let title = ''+modeType+' Stub for Bet#'+mod.objs.ticket_data.bet_info.unique_key;
								let modconfig = {
									target: 'print-ticket-modal',
									title: title,
									form: mod.objs.ticket_data.ticket,
								};

								mod.vars.utils.modal.prep(modconfig);
								mod.vars.utils.modal.show(modconfig);
								mod.ticket.print.init();
								mod.list.table.refresh();
							}
						}
					}, function(err){
						console.log(err);
						if (err.status == 406) {
							mod.vars.utils.prompt('error', err.responseJSON.message, 'Oops!');
						}
					});
				}
			},
			view: {
				init: function() {
					$(document).find('.view-bet').on('click', mod.ticket.view.load);
				},
				load: function(e) {
					e.preventDefault();
					let dis = $(this);
					let bkey = dis.data('bkey');
					let params = {};
					params['mode'] = 'view';
					params['_token'] = mod.vars.utils.vars.csrf;
					params['ishistory'] = mod.vars.ishistory;
					params['unique-key'] = bkey;
					mod.vars.utils.ajaxcall('POST', '/bet/ticket/claim', params).then(function(res) {
						console.log(res);
						if (res.success) {
							let modeType = 'View';
							mod.vars.utils.prompt('success', res.message, modeType+' payout');
							if (res.data.ticket) {
								mod.objs.ticket_data = res.data;
								mod.vars.user_role = res.data.role;
								let title = ''+modeType+' Stub for Bet#'+mod.objs.ticket_data.bet_info.unique_key;
								let modconfig = {
									target: 'print-ticket-modal',
									title: title,
									form: mod.objs.ticket_data.ticket,
								};

								mod.vars.utils.modal.prep(modconfig);
								mod.vars.utils.modal.show(modconfig);
								mod.ticket.print.init();
								mod.ticket.cancel.prep();
							}
						}
					}, function(err){
						console.log(err);
						if (err.status == 406) {
							mod.vars.utils.prompt('error', err.responseJSON.message, 'Oops!');
						}
					});
				}
			},
			// modal: {
			// 	init: function() {
			// 		mod.ticket.modal.when.hidden();
			// 		if (!mod.objs.ticket_modal_obj) {
			// 			mod.objs.ticket_modal_obj = new bootstrap.Modal(document.getElementById('print-ticket-modal'), {
			// 			  keyboard: false,
			// 			  backdrop: 'static'
			// 			});
						
			// 		}
			// 	},
			// 	prep: function(data) {
			// 		mod.vars.ticket_modal_el.find('[data-bs-dismiss="modal"]').on('click', mod.ticket.modal.hide);
			// 		mod.ticket.modal.reset();
			// 		mod.ticket.modal.init();
			// 		mod.vars.ticket_modal_el.find('.modal-title').html(data.title);
			// 		mod.vars.ticket_modal_el.find('.modal-body').html(data.ticket);
			// 	},
			// 	reset: function() {
			// 		mod.vars.ticket_modal_el.find('.modal-title').html('');
			// 		mod.vars.ticket_modal_el.find('.modal-body').html('<p>Modal body text goes here.</p>');
			// 	},
			// 	show: function() {
			// 		mod.objs.ticket_modal_obj.show();
			// 	},
			// 	hide: function() {
			// 		mod.objs.ticket_modal_obj.hide();
			// 	},
			// 	when: {
			// 		shown: function() {

			// 		},
			// 		hidden: function() {
			// 			document.getElementById('print-ticket-modal').addEventListener('hidden.bs.modal', function(event){
			// 				console.log(event);
			// 				console.log('Modal closed');
			// 			});
			// 		}
			// 	}
			// },
			print: {
				init: function() {
					if (mod.vars.print_bind == false) {
						$(document).find('.print-ticket').on('click', mod.ticket.print.start);	
						mod.vars.print_bind = true;
					}
				},
				start: function() {
					// var prtContent = document.getElementById('printable');
					var burl = mod.vars.ishistory === true ? '/print/view/history/' : '/print/view/';
					var WinPrint =
					window.open(burl+mod.objs.ticket_data.bet_info.unique_key,'','left=0,top=0');
					WinPrint.onload = function() {
						WinPrint.focus();
						WinPrint.print();
						WinPrint.onafterprint = function(){
						   console.log("Printing completed...");
						}

						setTimeout(function(){
							WinPrint.close();
						}, 700);
					};					
					// mod.vars.utils.ajaxcall('POST', '/print/ticket', {"unique-key":mod.objs.ticket_data.bet_info.unique_key}).then(function(res) {

					// 	if (res.success) {
					// 		mod.vars.utils.prompt('success', res.message, 'Print Status');
					// 	}

					// }, function(err){
					// 	console.log(err);
					// 	if (err.status == 406) {
					// 		mod.vars.utils.prompt('error', err.responseJSON.message, 'Oops!');
					// 		mod.payout.scanner.reset();
					// 	}
					// });

					// let title = 'Ticket for Bet#'+mod.objs.ticket_data.bet_info.unique_key;
					// let printable = $(document).find('#printable');
					// mod.vars.utils.print(printable, title, function() {
					// 	mod.vars.loaded_fight_table.ajax.reload();						
					// });					
				}
			}
		}
	};
	mod.init();
	return mod;
})();
