"use strict";
(function(){
	var mod = {
		vars: {
			date_input: $(document).find('[name="cashflow-date"]'),
			cashflow_table_list: $(document).find('#cashflow-table-list'),
			print_holder: $(document).find('#print-holder'),
			utils: null,
			date_set: null,
			dt_btn: {
				file: $(document).find('.btn-export-report'),
				print: $(document).find('.btn-print-report'),
				cashin: $(document).find('.btn-cash-in'),
				cashout: $(document).find('.btn-cash-out'),
			},
			print_bind: false,
			user_role: 'cashier',
			ishistory: null
		},
		objs: {
			ctl_obj: null,
			cashflow_modal_obj: null,
		},
		init: function() {
			mod.vars.utils = window.bsutils;
			mod.vars.ishistory = typeof ishistory != 'undefined' ? ishistory : false;
			mod.list.init();
			mod.buttons.init();
		},
		list: {
			init: function() {
				mod.list.date.init();
			},
			date: {
				init: function() {
					mod.vars.date_input.on('change', mod.list.date.onchange);
					mod.list.table.refresh();
					// mod.list.date.nowdate();
				},
				nowdate: function() {
					let now = new Date('2022-10-03');
					let date = now.toLocaleDateString("en-US",{year:"numeric",month:"2-digit",day:"2-digit"});
					let parseDate = date.split('/');
					let formattedDate = parseDate[2]+'-'+parseDate[0]+'-'+parseDate[1];
					mod.vars.date_set = formattedDate;
					mod.vars.date_input.val(formattedDate);
					mod.list.table.refresh();
				},
				onchange: function() {
					console.log($(this).val());
					mod.vars.date_set = $(this).val();
					mod.list.table.refresh();
				},
				latest: function() {
					console.log(mod.vars.date_set);
					return mod.vars.date_set;
				}
			},
			table: {
				refresh: function() {
					if (mod.objs.ctl_obj == null) {
						mod.list.table.load();
					} else {
						console.log(mod.objs.ctl_obj.ajax);
						mod.objs.ctl_obj.ajax.reload();
					}
				},
				load: function() {
					let server = window.location.origin + (mod.vars.ishistory == true ? '/datatable/cashflow-list/history' : '/datatable/cashflow-list');
					console.log(server);
					mod.objs.ctl_obj = mod.vars.cashflow_table_list.DataTable({
						// searching: false,
						paging: true,
						destroy: true,
						processing: true,
	        			serverSide: true,
	        			ajax: {
	                        url: server,
	                        data: function(d) {
	                        	d.ishistory = mod.vars.ishistory;
	                        	d._token = mod.vars.utils.vars.csrf;
	                        	d.date = mod.list.date.latest();
	                        },
	                        type: 'POST',
	                    },
	        			order: [[0, 'desc']],
	                    columnDefs: [{ "targets": "_all", "defaultContent": "" }],
	                    columns: [
	                    	{ "data":"cid", "title":"ID", "orderable":true, "searchable":false },
	                    	{ "data":"user", "title":"User", "orderable":true, "searchable":true },
	                    	{ "data":"user_cashedin", "title":"User In", "orderable":true, "searchable":true },
	                    	{ "data":"in_amount", "title":"In Amount", "orderable":true, "searchable":true },
	                    	{ "data":"cashin_date", "title":"In Date", "orderable":true, "searchable":true },
	                    	{ "data":"user_cashedout", "title":"User Out", "orderable":true, "searchable":true },
	                    	{ "data":"cashout_date", "title":"Out Date", "orderable":true, "searchable":true },
	                    	{ "data":"out_amount", "title":"Out Amount", "orderable":true, "searchable":true },
	                    	{ "data":"total_bets", "title":"Total Bets", "orderable":true, "searchable":true },
	                    	{ "data":"total_payout", "title":"Total Payout", "orderable":true, "searchable":true },
	                    	{ "data":"cancelled_payout", "title":"Cancelled Payout", "orderable":true, "searchable":true },
	                    	{ "data":"draw_payout", "title":"Draw Payout", "orderable":true, "searchable":true },
	                    	{ "data":"money_onhand", "title":"On-hand", "orderable":true, "searchable":true },
	                    	{ 
	                    		"data":"cid", "title":"Actions", "orderable":false, "searchable":false,
	                    		render: function(data, type, row) {
									return '<button href="#" class="btn btn-sm cashflow-print btn-success" data-cid="'+data+'"><i class="fa fa-print"></i></button>';
	                    		} 

	                    	}
	                    ],
	                    initComplete: function(distable, data){
	                    },
	                    drawCallback: function() {
	                    	mod.buttons.proof_print.init();
	                    }
					});	
				}
			},
		},
		buttons: {
			init: function() {
				mod.vars.dt_btn.file.on('click', mod.buttons.export.file.request);
				mod.vars.dt_btn.print.on('click', mod.buttons.print);
				mod.vars.dt_btn.cashin.on('click', mod.buttons.cashin);
				mod.vars.dt_btn.cashout.on('click', mod.buttons.cashout);
				mod.buttons.submit.init();
				mod.buttons.proof_print.init();
			},
			export: {
				file: {
					request: function(e) {
						e.preventDefault();
						if (mod.objs.ctl_obj.data().length > 0) {
							let dtparams = mod.objs.ctl_obj.ajax.params();
							dtparams.export_type = $(this).data('type');
							console.log(dtparams);
							mod.vars.utils.ajaxcall('POST', '/downloads/cashflow/request', dtparams).then(function(res){
								if (res.success) {
									if (res.data.new_window) {
										window.open(res.data.new_window);
									}
								}
							}, function(err) {
								console.log(err);
							});					
						} else {
							mod.vars.utils.prompt('error', 'Sorry! There is nothing to export on this current table.', 'Export table to PDF');
						}
					},
					download: function(response) {
						var blob = new Blob([response], {type: 'application/pdf'})
						var url = URL.createObjectURL(blob);
						location.assign(url);
					} 
				}
			},
			print: function(e) {
				e.preventDefault();
				if (mod.objs.ctl_obj.data().length > 0) {
					mod.vars.print_holder.html('');
					let dtparams = mod.objs.ctl_obj.ajax.params();
					mod.vars.utils.ajaxcall('POST', '/renders/cashflow/list/table', dtparams).then(function(res){
						if (res.success) {
							if (res.data.printable) {
								mod.vars.print_holder.append(res.data.printable);
								let printable = $(document).find('#printable-table');
								mod.vars.utils.print(printable, res.data.title, function() {
									mod.vars.print_holder.html('');
								});
							}
						}
					}, function(err) {
						console.log(err);
						mod.vars.print_holder.html('');
					});					
				} else {
					mod.vars.utils.prompt('error', 'Sorry! There is nothing to print on this current table.', 'Print table');
				}
			},
			cashin: function(e) {
				e.preventDefault();
				mod.vars.utils.ajaxcall('GET', '/cashflow/in', {}).then(function(res) {
					console.log(res);
					if (res.success) {
						mod.buttons.toggle.modal_buttons('cashflow');
						if (res.data.form) {
							let modconfig = {
								target: 'cashflow-modal',
								title: 'Cashing In',
								form: res.data.form,
							};

							mod.vars.utils.modal.prep(modconfig);
							mod.vars.utils.modal.show(modconfig);
						}
					}
				}, function(err){
					console.log(err);
					if (err.status == 406) {
						mod.vars.utils.prompt('error', err.responseJSON.message, 'Oops!');
					}
				});
			},
			cashout: function(e) {
				e.preventDefault();
				if (mod.objs.ctl_obj.data().length > 0) {
					mod.vars.utils.ajaxcall('GET', '/cashflow/out', {}).then(function(res) {
						console.log(res);
						if (res.success) {
							mod.buttons.toggle.modal_buttons('cashflow');
							if (res.data.form) {
								let modconfig = {
									target: 'cashflow-modal',
									title: 'Cashing Out',
									form: res.data.form,
								};

								mod.vars.utils.modal.prep(modconfig);
								mod.vars.utils.modal.show(modconfig);
							}
						}
					}, function(err){
						console.log(err);
						if (err.status == 406) {
							mod.vars.utils.prompt('error', err.responseJSON.message, 'Oops!');
						}
					});
				} else {
					mod.vars.utils.prompt('error', 'Sorry! There is nothing to cashout.', 'Print table');
				}
			},
			submit: {
				init: function() {
					if ($(document).find('.cashflow-submit').hasClass('event_loaded') == false) {
						$(document).find('.cashflow-submit').on('click', mod.buttons.submit.process);	
						$(document).find('.cashflow-submit').addClass('event_loaded');
					}
				},
				process: function(e) {
					e.preventDefault();
					let dis = $(this);
					let modalform = dis.closest('.modal').find('form');
					mod.vars.utils.validate(modalform).then(function(res) {
						mod.vars.utils.ajaxcall('POST', '/cashflow/process', modalform.serialize()).then(function(res) {
							console.log(res);
							if (res.success) {
								mod.buttons.toggle.modal_buttons('proof');
								if (res.data.proof) {
									let modconfig = {
										target: 'cashflow-modal',
										title: 'Latest Cash Flow',
										form: res.data.proof,
									};

									mod.vars.utils.modal.prep(modconfig);
									mod.vars.utils.modal.show(modconfig);
									mod.objs.ctl_obj.ajax.reload();
								}
							}
						}, function(err){
							console.log(err);
							if (err.status == 406) {
								mod.vars.utils.prompt('error', err.responseJSON.message, 'Oops!');
								mod.payout.scanner.reset();
							}
						});
					});
				}
			},
			toggle: {
				modal_buttons: function(mode) {
					if (mode == 'proof') {
						$(document).find('.modal .cashflow-submit').hide();
						$(document).find('.modal .cashflow-print').show();
					} else {
						$(document).find('.modal .cashflow-submit').show();
						$(document).find('.modal .cashflow-print').hide();
					}
				}
			},
			proof_print: {
				init: function() {
					$(document).find('.cashflow-print').each(function(){
						if ($(this).hasClass('event_loaded') == false) {
							$(this).on('click', mod.buttons.proof_print.start);
							$(this).addClass('event_loaded');
						}
					});
				},
				start: function(e) {
					e.preventDefault();
					let dis = $(this);
					let cid = dis.data('cid');
					console.log(cid);
					if (cid == 'undefined' || cid == undefined) {
						if (dis.closest('.modal').find('input[name="cid"]').length > 0) {
							cid = dis.closest('.modal').find('input[name="cid"]').val();
						}
					}
					var cfurl = mod.vars.ishistory === true ? '/print/cashflow/history/' : '/print/cashflow/';
					var WinPrint = window.open(cfurl+cid,'','left=0,top=0');
					WinPrint.onload = function() {
						WinPrint.focus();
						WinPrint.print();
						WinPrint.onafterprint = function(){
						   console.log("Printing completed...");
						   WinPrint.close();
						}

						setTimeout(function(){
							WinPrint.close();
						}, 700);	
					};
				},
			},


		},
	};
	mod.init();
	return mod;
})();
