"use strict";
(function(){
	var mod = {
		vars: {
			form: null,
			action: '',
			utils: null,
            promo_table: $(document).find('#promo-table-list')
		},
		objs: {
            html5scanner: null,
        },
		init: function() {
			mod.vars.utils = window.bsutils;
            mod.table.load();
		},
        table: {
            load: function() {
                let server = window.location.origin+'/promo/list';
                console.log(server);
                mod.vars.promo_table = mod.vars.promo_table.DataTable({
                    searching: true,
                    processing: true,
                    serverSide: true,
                    ajax: {
                        url: server,
                        data: {_token:mod.vars.utils.vars.csrf},
                        type: 'POST',
                    },
                    order: [[0, 'desc']],
                    columnDefs: [{ "targets": "_all", "defaultContent": "" }],
                    columns: [
                        { "data" : "promo_name", "title" : "Name", "orderable": true, "searchable": true },
                        { "data" : "promo_details", "title" : "Details", "orderable": true, "searchable": true },
                        { "data" : "start", "title" : "Start", "orderable": true, "searchable": true },
                        { "data" : "end", "title" : "End", "orderable": true, "searchable": true },
                        { "data" : "point_qualifications", "title" : "Points Needed", "orderable": true, "searchable": true },
                        { "data" : "broadcast", "title" : "Broadcast", "orderable": true, "searchable": true },
                        {
                            "data":"actions", "title":"Actions", "orderable":false, "searchable":false,
                            render: function(data, type, row) {
                                let delete_btn = row.role == 'admin' ? '' : '&nbsp;<button type="button" class="ml btn btn-sm btn-danger delete-promo" data-pid="'+row.id+'"><i class="fa fa-trash"></i></button>';
                                let claim_btn = row.role == 'admin' ? '' : '&nbsp;<button type="button" class="ml mb-2 btn btn-sm btn-warning claim-promo" data-pid="'+row.id+'">Claim</button>';
                                return '<button type="button" class="btn btn-sm btn-success call-promo-form mb-2" data-pid="'+row.id+'">Edit</button>'+claim_btn+delete_btn;
                            }
                        }
                    ],
                    initComplete: function(distable, data){
                    },
                    drawCallback: function() {
                        mod.form.init_btn();
                    }
                });
            },
            customer: {
                load: function() {
                    mod.vars.customer_table = $(document).find('#customer-table-list');
                    let server = window.location.origin+'/customer/list';

                        mod.vars.customer_table = mod.vars.customer_table.DataTable({
                            searching: true,
                            processing: true,
                            serverSide: true,
                            ajax: {
                                url: server,
                                data: {_token:mod.vars.utils.vars.csrf},
                                type: 'POST',
                            },
                            order: [[1, 'desc']],
                            columnDefs: [{ "targets": "_all", "defaultContent": "" }],
                            columns: [
                                { "data" : "email",        "title" : "Email",           "orderable": true, "searchable": true },
                                { "data" : "name",        "title" : "Name",           "orderable": true, "searchable": true },
                                {
                                    "data":"actions", "title":"Actions", "orderable":false, "searchable":false,
                                    render: function(data, type, row) {
                                        let rowstr = JSON.stringify(row);
                                        return '<button type="button" class="btn btn-sm btn-success modal-select-user" data-uid="'+row.id+'" data-name="'+row.name+'"><i class="fa fa-check"></i></button>';
                                    }
                                }
                            ],
                            initComplete: function(distable, data){
                            },
                            drawCallback: function() {
                                mod.form.search.init_btn();
                            }
                        });
                }
            }
        },
        form: {
            init_btn: function() {
				$(document).find('.call-promo-form').each(function(){
					let dis = $(this);
					if (dis.hasClass('hasEvent')==false) {
						dis.on('click', mod.form.call);
						dis.addClass('hasEvent');
					}
				});

				$(document).find('.delete-promo').each(function(){
					let dis = $(this);
					if (dis.hasClass('hasEvent')==false) {
						dis.on('click', mod.form.delete);
						dis.addClass('hasEvent');
					}
				});

                $(document).find('.claim-promo').each(function(){
					let dis = $(this);
					if (dis.hasClass('hasEvent')==false) {
						dis.on('click', mod.form.claim);
						dis.addClass('hasEvent');
					}
				});

				mod.form.load();
			},
			delete: function(e) {
				e.preventDefault();
				let dis = $(this);
				let pid = typeof dis.data('pid') != 'undefined' ? dis.data('pid') : 0 ;
				if (pid > 0) {
					if (confirm("Are you sure you want to delete this promo?") == true) {
						mod.vars.utils.ajaxcall('POST', '/promo/delete', {"promo_id":pid}).then(function(res) {
							console.log(res);
							if (res.success) {
								mod.vars.utils.prompt('success', res.message, 'Promo Removal');
								mod.vars.promo_table.ajax.reload();
							}
						}, function(err){
							console.log(err);
							if (err.status == 406) {
								mod.vars.utils.prompt('error', err.responseJSON.message, 'Oops!');
                                mod.vars.utils.prompt('error', err.responseJSON.data, 'Oops!');
							}
						});
					}
				}
			},
            claim: function(e) {
				e.preventDefault();
				let dis = $(this);
				let pid = typeof dis.data('pid') != 'undefined' ? dis.data('pid') : 0 ;
				if (pid > 0) {
					mod.vars.utils.ajaxcall('GET', '/promo/form/claim/'+pid, {}).then(function(res) {
                        console.log(res);
                        if (res.success) {
                            console.log(res);
                            let mconfig = {
                                target: 'promo-modal',
                                title: pid > 0 ? 'Claim Promo' : 'Claim Promo',
                                form: res.data.user_form,
                                save_btn: {
                                    classes: 'start-claim-promo',
                                    text: 'Start Claim',
                                }
                            };
                            mod.vars.utils.modal.prep(mconfig);
                            mod.vars.utils.modal.show();
                            mod.table.customer.load();
                            mod.form.start_claim.init_btn();
                            // if (mod.objs.html5scanner != null) {
                            //     mod.objs.html5scanner.clear();
                            // }
                            // setTimeout(function(){
                            //     mod.form.qr.scanner.init();
                            // }, 500);
                        }
                    }, function(err){
                        console.log(err);
                        if (err.status == 406) {
                            mod.vars.utils.prompt('error', err.responseJSON.message, 'Oops!');
                            mod.vars.utils.prompt('error', err.responseJSON.data, 'Oops!');
                        }
                    });
				}
			},
			call: function(e) {
				e.preventDefault();
				let dis = $(this);
				let pid = typeof dis.data('pid') != 'undefined' ? dis.data('pid') : 0 ;
				mod.vars.utils.ajaxcall('GET', '/promo/form/'+pid, {}).then(function(res) {
					if (res.success) {
						console.log(res);
						let mconfig = {
							target: 'create-promo-modal',
							title: pid > 0 ? 'Edit Promo' : 'Create Promo',
							form: res.data.user_form
						};
						mod.vars.utils.modal.prep(mconfig);
						mod.vars.utils.modal.show();
					}
				}, function(err){
					console.log(err);
					if (err.status == 406) {
						mod.vars.utils.prompt('error', err.responseJSON.message, 'Oops!');
                        mod.vars.utils.prompt('error', err.responseJSON.data, 'Oops!');
					}
				});
			},
			load: function() {
				$(document).find('.save-user').each(function(){
					let dis = $(this);
					if (dis.hasClass('hasEvent')==false) {
						dis.on('click', mod.form.submit);
						dis.addClass('hasEvent');
					}
				});
			},
			submit: function(e) {
				e.preventDefault();
				let dis = $(this);
				let form = dis.closest('.modal').find('form');
				let dataset = form.serialize();
                let mode = form.find('[name="mode"]').val() == 'edit' ? 'update' : 'create';
				mod.vars.utils.ajaxcall('POST', '/promo/'+mode, dataset).then(function(res){
					if (res.success) {
						mod.vars.utils.prompt('success', res.message, 'Promo');
						mod.vars.utils.modal.hide();
						mod.vars.promo_table.ajax.reload();
					}
				}, function(err){
					console.log(err);
					if (err.status == 406) {
						mod.vars.utils.prompt('error', err.responseJSON.message, 'Oops!');
                        mod.vars.utils.prompt('error', err.responseJSON.data, 'Oops!');
					}
				});
			},
            start_claim: {
                init_btn: function() {
                    $(document).find('.start-claim-promo').each(function(){
                        let dis = $(this);
                        if (dis.hasClass('hasEvent')==false) {
                            dis.on('click', mod.form.start_claim.submit);
                            dis.addClass('hasEvent');
                        }
                    });
                },
                submit: function(e) {
                    e.preventDefault();
                    let dis = $(this);
                    let form = dis.closest('.modal').find('#claim-form');
                    let dataset = form.serialize();
                    mod.vars.utils.ajaxcall('POST', '/promo/start_claim', dataset).then(function(res) {
                        console.log(res);
                        if (res.success) {
                            mod.vars.utils.prompt('success', res.message, 'Promo Claim');
                            mod.vars.utils.modal.hide();
                        }
                    }, function(err){
                        console.log(err);
                        if (err.status == 406) {
                            mod.vars.utils.prompt('error', err.responseJSON.message, 'Oops!');
                            mod.vars.utils.prompt('error', err.responseJSON.data, 'Oops!');
                        }
                    });
                }

            },
            search: {
                init_btn: function() {
                    $(document).find('.modal-select-user').each(function(){
                        let dis = $(this);
                        if (dis.hasClass('hasEvent')==false) {
                            dis.on('click', mod.form.search.select_btn);
                            dis.addClass('hasEvent');
                        }
                    });
                },
                select_btn: function(e) {
                    e.preventDefault();
                    let dis = $(this);
                    let customer_id = dis.data('uid');
                    if (customer_id != undefined) {
                        mod.form.search.call_user_fields(customer_id);
                    }
                    // console.log(customer_id);
                    // let name = dis.data('name');
                    // console.log(name);
                    // let form = $(document).find('#user-form');
                    // form.find('.call-claim-form').attr('data-uid', customer_id);
                    // form.find('#customer_name').val(name);
                },
                call_user_fields: function(uid) {
                    if (uid > 0) {
                        mod.vars.utils.ajaxcall('GET', '/user/form/claim/'+uid, {}).then(function(res) {
                            console.log(res);
                            if (res.success) {
                                $(document).find('#customer-data').html(res.data.user_form);
                                $(document).find('.modal.show .modal-body').scrollTop(-1000);

                            }
                        }, function(err){
                            console.log(err);
                            if (err.status == 406) {
                                mod.vars.utils.prompt('error', err.responseJSON.message, 'Oops!');
                                mod.vars.utils.prompt('error', err.responseJSON.data, 'Oops!');
                            }
                        });
                    }
                }
            },
            // qr: {
            //     scanner:  {
            //         init: function() {
            //             mod.objs.html5scanner = new Html5QrcodeScanner(
            //                 "preview", { fps: 10, qrbox: 250, rememberLastUsedCamera: false });
            //             mod.objs.html5scanner.render(mod.form.qr.scanner.onSuccess);
            //         },
            //         onSuccess: function(decodedText, decodedResult) {
            //             if (decodedText != '' && mod.vars.scan_ajax == false) {
            //                 console.log(`Scan result: ${decodedText}`, decodedResult);
            //                 mod.vars.scan_ajax = true;
            //                 mod.form.transaction.ajaxcall('scan', decodedText, 0).then(function(res){
            //                     mod.objs.html5scanner.clear();
            //                 });

            //             }
            //         },
            //     }
            // }
        }
	};
	mod.init();
	return mod;
})();
