"use strict";
(function(){
	var mod = {
		vars: {
			csrf: $(document).find('meta[name="csrf-token"]').attr('content'),
			modal_conf: {},
			preloader: $(document).find('.preloader')
		},
		objs: {
			modal_el: null,
			modal_ob: null,
		},
		init: function() {
			mod.vars.preloader.hide();
		},
		money_format: function(amount, decimal_place, comma_place) {
            var re = '\\d(?=(\\d{' + (comma_place || 3) + '})+' + (decimal_place > 0 ? '\\.' : '$') + ')';
                return parseFloat(amount).toFixed(Math.max(0, ~~decimal_place)).replace(new RegExp(re, 'g'), '$&,');
        },
		number_format: function(value) {
                var valid = parseFloat(value);
                var num = !valid ? '0.00' : valid.toFixed(0);
                return num.toString().replace(/(\d)(?=(\d{3})+(?!\d))/g, '$1,');
        },
        plain_number_format: function(value) {
            var valid = parseFloat(value);
            var num = !valid ? '0' : valid.toFixed(0);
            return num.toString().replace(/(\d)(?=(\d{3})+(?!\d))/g, '$1,');
        },
		print: function(el, title, afterPrint) {
			el.printThis({
				pageTitle: title,
				importCSS: true,                // import parent page css
				importStyle: true,              // import style tags
				printContainer: true,
				printDelay: 300,
				afterPrint: afterPrint,
			});
		},
        ajaxupload: function(path, data) {
            mod.vars.preloader.show();
            console.log(typeof data);

			//If serialize string add _token via concatination
			if (typeof data == 'object') {
				data._token = mod.vars.csrf;
			}

			if (typeof data == 'string') {
				data += '&_token='+mod.vars.csrf;
			}

			return new Promise(function(resolve, reject) {
	            $.ajax({
	                url: path,
	                type: 'POST',
	                cache: false,
	                data: data,
	                datatype: "json",
                    contentType: false,
                    processData: false,
	                success: function(response) {
	                	mod.vars.preloader.hide();
	                    return resolve(response);
	                },
	                error: function(err) {
	                	mod.vars.preloader.hide();
	                	return reject(err);
	                }
	            });
			});
        },
		ajaxcall: function(method, path, data) {

			mod.vars.preloader.show();
            console.log(typeof data);

			//If serialize string add _token via concatination
			if (typeof data == 'object') {
				data._token = mod.vars.csrf;
			}

			if (typeof data == 'string') {
				data += '&_token='+mod.vars.csrf;
			}

			return new Promise(function(resolve, reject) {
	            $.ajax({
	                url: path,
	                type: method,
	                cache: false,
	                data: data,
	                datatype: "json",
	                success: function(response) {
	                	mod.vars.preloader.hide();
	                    return resolve(response);
	                },
	                error: function(err) {
                        if (err.status == 401) {
                            window.location.reload();
                        }
	                	mod.vars.preloader.hide();
	                	return reject(err);
	                }
	            });
			});
        },
        prompt: function(type, message, title) {
        	let modtitle = typeof title == 'undefined' ? type.toUppercase() : title;
        	switch(type) {
				case 'warning':
					toastr.warning(message, modtitle);
				break;
				case 'error':
					toastr.error(message, modtitle);
				break;
				case 'success':
					toastr.success(message, modtitle);
				break;
				default:
					toastr.info(message, modtitle);
			}
        },
        showerror: function(_form, message) {
        	_form.append('<div class="alert alert-danger my-3">'+message+'</div>');
        },
        showprompt: function(_form, message) {
        	_form.append('<div class="success success-danger my-3">'+message+'</div>');
        },
		reset: function(_form) {
			_form.find('.alert').remove();
			_form.find('.required').removeClass('is-invalid');
			_form.find('.required').closest('div').find('.badge').remove();
		},
		validate: function(_form) {
			mod.reset(_form);
			let validForm = true;
			return new Promise(function(resolve, reject) {
				_form.find('.required').each(function(){
					if ($(this).val() == '') {
						$(this).addClass('is-invalid');
						let name = $(this).attr('name');
						$(this).closest('div').append('<span class="badge badge-danger">'+name.toUpperCase()+' is invalid.</span>');
						validForm = false;
					}
				});
				return resolve(validForm);
			});
		},
		redirect: function(path) {
			window.location.href = window.location.origin + '/' + path;
		},
		removeClassStartingWith: function(el, filter) {
		    el.removeClass(function(index, className) {
		        return (className.match(new RegExp("\\S*" + filter + "\\S*", 'g')) || []).join(' ')
		    });
		},
		toTitleCase: function(str) {
		  return str.replace(/\w\S*/g, function(txt){return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase();});
		},
		modal: {
			init: function() {
				mod.modal.when.hidden();
				mod.objs.modal_ob = new bootstrap.Modal(document.getElementById(mod.vars.modal_conf.target), {
                    keyboard: false,
                    backdrop: 'static'
                });
			},
			prep: function(configs) {
				mod.vars.modal_conf = configs;
                // console.log(mod.vars.modal_conf);
				mod.objs.modal_el = $(document).find('#'+mod.vars.modal_conf.target);
				mod.objs.modal_el.find('[data-bs-dismiss="modal"]').on('click', mod.modal.hide);
				mod.modal.reset();
				mod.modal.init();
                // console.log(mod.objs.modal_ob);
				mod.objs.modal_el.find('.modal-title').html(mod.vars.modal_conf.title);
				mod.objs.modal_el.find('.modal-body').html(mod.vars.modal_conf.form);
                if (mod.vars.modal_conf.hide_btns != undefined) {
                    mod.objs.modal_el.find('.modal-footer button').hide();
                }
                if (mod.vars.modal_conf.save_btn != undefined) {
                    let btnPrimary = mod.objs.modal_el.find('.btn-primary');
                    btnPrimary.addClass(mod.vars.modal_conf.save_btn.classes);
                    btnPrimary.html(mod.vars.modal_conf.save_btn.text);
                }
                if (mod.vars.modal_conf.close_btn != undefined) {
                    let btnPrimary = mod.objs.modal_el.find('.btn-primary');
                    btnPrimary.addClass(mod.vars.modal_conf.close_btn.classes);
                    btnPrimary.html(mod.vars.modal_conf.close_btn.text);
                }
			},
			reset: function() {
				mod.objs.modal_el.find('.modal-title').html('');
				mod.objs.modal_el.find('.modal-body').html('<p>Modal body text goes here.</p>');
                mod.objs.modal_el.find('.modal-footer button').show();
			},
			show: function() {
				mod.objs.modal_ob.show();
                mod.objs.modal_el.find('input:eq(0)').focus();
			},
			hide: function() {
                if (mod.objs.modal_ob != null) {
                    mod.objs.modal_ob.hide();
                }
			},
			when: {
				shown: function() {

				},
				hidden: function() {
					document.getElementById(mod.vars.modal_conf.target).addEventListener('hidden.bs.modal', function(event){
						console.log(event);
						console.log('Modal closed');
					});
				}
			}
		},
		cookie: {
			set: function(name,value,days) {
			    var expires = "";
			    if (days) {
			        var date = new Date();
			        date.setTime(date.getTime() + (days*24*60*60*1000));
			        expires = "; expires=" + date.toUTCString();
			    }
			    document.cookie = name + "=" + (value || "")  + expires + "; path=/";
			},
			get: function(name) {
			    var nameEQ = name + "=";
			    var ca = document.cookie.split(';');
			    for(var i=0;i < ca.length;i++) {
			        var c = ca[i];
			        while (c.charAt(0)==' ') c = c.substring(1,c.length);
			        if (c.indexOf(nameEQ) == 0) return c.substring(nameEQ.length,c.length);
			    }
			    return null;
			},
			erase: function(name) {
			    document.cookie = name +'=; Path=/; Expires=Thu, 01 Jan 1970 00:00:01 GMT;';
			}
		}
	};
	mod.init();
	window.bsutils = mod;
	return mod;
})();
