<?php

use App\Http\Controllers\API\BroadcastController;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;

use App\Http\Controllers\API\SecurityController;
use App\Http\Controllers\API\UserController;
use App\Http\Controllers\API\CashierController;
use App\Http\Controllers\API\CustomerController;
use App\Http\Controllers\API\TransactionController;
use App\Http\Controllers\API\NotificationController;
use App\Http\Controllers\API\PromoController;
use App\Http\Controllers\API\ClaimedController;
use App\Http\Controllers\API\ServiceController;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider and all of them will
| be assigned to the "api" middleware group. Make something great!
|
*/

Route::middleware('auth:sanctum')->group(function(){
    Route::prefix('user')->group(function(){
        Route::get('current', [UserController::class, 'currentUser']);
        Route::post('update', [UserController::class, 'userUpdate']);
        Route::post('avatar', [UserController::class, 'uploadAvatar']);
        Route::post('status', [UserController::class, 'updateStatus']);
        // Route::get('company', [UserController::class, 'userCompany']);
        Route::get('{id}/full', [UserController::class, 'fullProfile']);
        Route::get('{id}/profile', [UserController::class, 'userProfile']);
    });

    Route::prefix('cashier')->group(function(){
        Route::get('list', [CashierController::class, 'list']);
        Route::get('detail/{id}', [CashierController::class, 'detail']);
        Route::post('create', [CashierController::class, 'create']);
        Route::post('update', [CashierController::class, 'update']);
        Route::post('delete', [CashierController::class, 'delete']);
    });

    Route::prefix('customer')->group(function(){
        Route::get('list', [CustomerController::class, 'list']);
        Route::get('detail/{id}', [CustomerController::class, 'detail']);
        Route::post('create', [CustomerController::class, 'create']);
        Route::post('update', [CustomerController::class, 'update']);
        Route::post('delete', [CustomerController::class, 'delete']);
    });

    Route::prefix('transaction')->group(function(){
        Route::get('list', [TransactionController::class, 'list']);
        Route::get('detail/{id}', [TransactionController::class, 'detail']);
        Route::post('create', [TransactionController::class, 'create']);
        Route::post('update', [TransactionController::class, 'update']);
        Route::post('delete', [TransactionController::class, 'delete']);
    });

    Route::prefix('notification')->group(function(){
        Route::get('list', [NotificationController::class, 'list']);
        Route::get('detail/{id}', [NotificationController::class, 'detail']);
        Route::post('create', [NotificationController::class, 'create']);
        Route::post('update', [NotificationController::class, 'update']);
        Route::post('delete', [NotificationController::class, 'delete']);
        Route::post('poster', [NotificationController::class, 'uploadPoster']);
        Route::post('broadcast', [NotificationController::class, 'broadcast']);
    });

    Route::prefix('claimed')->group(function(){
        Route::post('list', [ClaimedController::class, 'list']);
    });

    Route::prefix('promo')->group(function(){
        Route::post('list', [PromoController::class, 'list']);
        Route::get('detail/{id}', [PromoController::class, 'detail']);
        Route::post('create', [PromoController::class, 'create']);
        Route::post('update', [PromoController::class, 'update']);
        Route::post('delete', [PromoController::class, 'delete']);
        Route::post('poster', [PromoController::class, 'uploadPoster']);
        Route::post('broadcast', [PromoController::class, 'broadcast']);
    });

    Route::prefix('service')->group(function(){
        Route::post('list', [ServiceController::class, 'list']);
        Route::get('detail/{id}', [ServiceController::class, 'detail']);
        Route::post('create', [ServiceController::class, 'create']);
        Route::post('update', [ServiceController::class, 'update']);
        Route::post('delete', [ServiceController::class, 'delete']);
    });

    Route::prefix('broadcast')->group(function(){
        Route::get('email', [BroadcastController::class, 'broadcastViaEmail']);
    });

    // Route::post('send_invites', [SecurityController::class, 'invites']);
    // Route::prefix('company')->group(function(){
    //     Route::get('is/{id}', [CompanyController::class, 'readCompany']);
    //     Route::post('create', [CompanyController::class, 'create']);
    //     Route::post('edit', [CompanyController::class, 'edit']);
    //     Route::post('delete', [CompanyController::class, 'delete']);
    //     Route::post('delete-all', [CompanyController::class, 'deleteAll']);
    // });
    // Route::prefix('agent')->group(function(){
    //     Route::get('is/{companyid}/{id}', [AgentController::class, 'readAgent']);
    //     Route::post('create', [AgentController::class, 'create']);
    //     Route::post('edit', [AgentController::class, 'edit']);
    //     Route::post('delete', [AgentController::class, 'delete']);
    // });
    // Route::prefix('call')->group(function(){
    //     Route::get('is/{company_unique_key}/{id}', [CallController::class, 'readCall']);
    //     Route::get('list/{company_unique_key}', [CallController::class, 'readList']);
    //     Route::post('create', [CallController::class, 'create']);
    //     Route::post('edit', [CallController::class, 'edit']);
    //     Route::post('delete', [CallController::class, 'delete']);
    //     Route::get('broadcast', [CallController::class, 'broadcastCall'])->middleware('throttle:60,1');
    //     Route::post('set', [CallController::class, 'setCall']);
    //     Route::get('agent/{id}', [CallController::class, 'agentCalls']);
    // });
    // Route::prefix('media')->group(function(){
    //     Route::get('item/images/{type}/{id}', [MediaController::class, 'itemImages'])->name('media-images-item');
    //     Route::get('item/videos/{id}', [MediaController::class, 'itemVideos'])->name('media-videos-item');
    //     Route::post('upload', [MediaController::class, 'upload'])->name('media-upload');
    //     Route::post('delete', [MediaController::class, 'delete'])->name('media-delete');
    // });
});

Route::post('login', [SecurityController::class, 'loginUser']);
Route::options('login', [SecurityController::class, 'loginUser']);
Route::post('signup', [SecurityController::class, 'registerUser']);
Route::options('signup', [SecurityController::class, 'registerUser']);
Route::post('verify_code', [SecurityController::class, 'verifyCode']);
Route::post('request_reset', [SecurityController::class, 'requestReset']);
Route::post('verify_reset', [SecurityController::class, 'verifyReset']);
Route::post('password_reset', [SecurityController::class, 'passwordReset']);
Route::get('unauthorized', [SecurityController::class, 'unauthorized'])->name('api-unauthorized');
// Route::get('validate_invites/{code}', [SecurityController::class, 'validateInvites']);
// Route::options('validate_invites/{code}', [SecurityController::class, 'validateInvites']);
// Route::post('google_signup', [SecurityController::class, 'googleRegisterUser']);
// Route::post('google_login', [SecurityController::class, 'googleLoginUser']);
