<?php

use App\Http\Controllers\API\BroadcastController;
use Illuminate\Support\Facades\Route;
use App\Http\Controllers\AuthController;
use App\Http\Controllers\UserController as UserViewController;
use App\Http\Controllers\API\UserController;
use App\Http\Controllers\API\SecurityController;
use App\Http\Controllers\API\CashierController;
use App\Http\Controllers\API\CustomerController;
use App\Http\Controllers\API\TransactionController;
use App\Http\Controllers\API\PromoController;
use App\Http\Controllers\API\ClaimedController;
use App\Http\Controllers\API\ServiceController;

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider and all of them will
| be assigned to the "web" middleware group. Make something great!
|
*/

Route::get('/', function () {
    return redirect('login');
});

Route::get('/home', function () {
    return redirect('dashboard');
});

Route::get('login', [AuthController::class, 'loginView'])->name('login');
Route::get('validate_reset/{token}', [AuthController::class, 'validateResetView'])->name('validate-reset');

Route::prefix('oauth')->group(function(){
    Route::post('login', [SecurityController::class, 'loginUser']);
    Route::options('login', [SecurityController::class, 'loginUser']);
    Route::post('signup', [SecurityController::class, 'registerUser']);
    Route::options('signup', [SecurityController::class, 'registerUser']);
    Route::post('verify_code', [SecurityController::class, 'verifyCode']);
    Route::post('request_reset', [SecurityController::class, 'requestReset']);
    Route::post('verify_reset', [SecurityController::class, 'verifyReset']);
    Route::post('password_reset', [SecurityController::class, 'passwordReset']);
    Route::get('unauthorized', [SecurityController::class, 'unauthorized'])->name('unauthorized');
});

Route::middleware('auth')->group(function(){

    Route::get('dashboard', [UserViewController::class, 'dashboardView'])->name('dashboard');
    Route::get('settings', [UserViewController::class, 'dashboardView'])->name('settings');

    Route::get('logout', [UserViewController::class, 'logout'])->name('logout');

    Route::prefix('user')->group(function(){
        Route::get('/', [UserViewController::class, 'dashboardView'])->name('dashboard');
        Route::prefix('profile')->group(function(){
            Route::get('/', [UserViewController::class, 'profileView'])->name('profile');
            Route::post('update', [UserController::class, 'profileUpdate'])->name('profile-update');
        });
        Route::post('update', [UserController::class, 'userUpdate']);
        Route::get('current', [UserController::class, 'currentUser']);
        Route::post('update', [UserController::class, 'userUpdate']);
        Route::post('avatar', [UserController::class, 'uploadAvatar']);
        Route::post('status', [UserController::class, 'updateStatus']);
        // Route::get('company', [UserController::class, 'userCompany']);
        Route::get('{id}/full', [UserController::class, 'fullProfile']);
        Route::get('{id}/profile', [UserController::class, 'userProfile']);
        Route::get('form/{uid}', [UserController::class, 'userForm'])->name('user-form');
        Route::get('form/claim/{uid}', [UserController::class, 'userClaimForm'])->name('user-claim-form');
    });

    Route::prefix('cashier')->group(function(){
        Route::get('/', [UserViewController::class, 'cashierView'])->name('cashiers');
        Route::post('list', [CashierController::class, 'list']);
        Route::get('detail/{id}', [CashierController::class, 'detail']);
        Route::post('create', [CashierController::class, 'create']);
        Route::post('update', [CashierController::class, 'update']);
        Route::post('delete', [CashierController::class, 'delete']);
    });

    Route::prefix('customer')->group(function(){
        Route::get('/', [UserViewController::class, 'customerView'])->name('customers');
        Route::post('list', [CustomerController::class, 'list']);
        Route::get('detail/{id}', [CustomerController::class, 'detail']);
        Route::post('create', [CustomerController::class, 'create']);
        Route::post('update', [CustomerController::class, 'update']);
        Route::post('delete', [CustomerController::class, 'delete']);
    });

    Route::prefix('transaction')->group(function(){
        Route::get('/', [UserViewController::class, 'transactionView'])->name('transactions');
        Route::post('list', [TransactionController::class, 'list']);
        Route::get('detail/{id}', [TransactionController::class, 'detail']);
        Route::post('create', [TransactionController::class, 'create']);
        Route::post('update', [TransactionController::class, 'update']);
        Route::post('delete', [TransactionController::class, 'delete']);
        Route::get('form/{uid}/{txnid}', [TransactionController::class, 'txnForm'])->name('txn-form');
        Route::get('scan/{encrypted}/{txnid}', [TransactionController::class, 'txnFormByScanner'])->name('txn-form-by-scanner');
        Route::get('mobile/form', [TransactionController::class, 'mobileForm'])->name('mobile-form');
        Route::post('mobile/verify', [TransactionController::class, 'mobileVerify'])->name('mobile-verify');
        Route::get('user/search', [TransactionController::class, 'userSearchForm'])->name('user-search-form');
        Route::get('qrpreview', [TransactionController::class, 'userQRForm'])->name('user-qr-form');
    });

    Route::prefix('promo')->group(function(){
        Route::get('/', [UserViewController::class, 'promoView'])->name('promos');
        Route::post('list', [PromoController::class, 'list']);
        Route::get('detail/{id}', [PromoController::class, 'detail']);
        Route::post('create', [PromoController::class, 'create']);
        Route::post('update', [PromoController::class, 'update']);
        Route::post('delete', [PromoController::class, 'delete']);
        Route::post('poster', [PromoController::class, 'uploadPoster']);
        Route::post('broadcast', [PromoController::class, 'broadcast']);
        Route::get('form/{pid}', [PromoController::class, 'promoForm'])->name('promo-form');
        Route::get('form/claim/{pid}', [PromoController::class, 'promoClaimForm'])->name('promo-form');
        Route::post('start_claim', [PromoController::class, 'startClaim'])->name('start-claim');
    });

    Route::prefix('claims')->group(function(){
        Route::get('/', [UserViewController::class, 'claimsView'])->name('claims');
        Route::post('list', [ClaimedController::class, 'list']);
    });

    Route::prefix('service')->group(function(){
        Route::get('/', [UserViewController::class, 'servicesView'])->name('services');
        Route::post('list', [ServiceController::class, 'list']);
        Route::get('detail/{id}', [ServiceController::class, 'detail']);
        Route::post('create', [ServiceController::class, 'create']);
        Route::post('update', [ServiceController::class, 'update']);
        Route::post('delete', [ServiceController::class, 'delete']);
        Route::get('form/{sid}', [ServiceController::class, 'servicesForm'])->name('services-form');
    });

    Route::prefix('broadcast')->group(function(){
        Route::get('email', [BroadcastController::class, 'broadcastViaEmail']);
        Route::get('detail/{id}', [BroadcastController::class, 'detail']);
        Route::post('readall', [BroadcastController::class, 'readAll']);
    });
});

Route::prefix('img')->group(function(){
    Route::get('test', [UserViewController::class, 'testImage']);
});
