<?php

namespace Tests\Feature\Main;

use Faker\Factory;
use Faker\Generator;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use Tests\TestCase;
use Laravel\Sanctum\Sanctum;

class CashierTest extends TestCase
{
    public function test_create_cashier_no_validation_true(): void
    {
        try {
            Sanctum::actingAs(\App\Models\User::where('email', env('TEST_USEREMAIL'))->first(),['*']);
            for ($i = 0; $i < 5; $i++) {
                $faker = Factory::create();
                //Create 4 cashiers
                $response = $this->json('POST', '/api/cashier/create', [
                    'email' => $faker->email,
                    'firstname' => $faker->firstName,
                    'lastname' => $faker->lastName,
                    'password' => 'demopass567',
                    'password_confirmation' => 'demopass567',
                    'account_validation' => 'no',
                    'newsletter_validation' => 'yes',
                ]);
                $response->assertStatus(200)->assertJson([
                    'success' => true,
                    'message' => 'Cashier account is created. Thanks.'
                    ]);
            }
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function test_create_cashier_with_validation_true(): void
    {
        try {
            Sanctum::actingAs(\App\Models\User::where('email', env('TEST_USEREMAIL'))->first(),['*']);
            for ($i = 0; $i < 5; $i++) {
                $faker = Factory::create();
                $payload = [
                    'email' => $faker->email,
                    'firstname' => $faker->firstName,
                    'lastname' => $faker->lastName,
                    'password' => 'demopass567',
                    'password_confirmation' => 'demopass567',
                    'account_validation' => 'yes',
                    'newsletter_validation' => 'no',
                ];
                //Create 4 cashiers
                $response = $this->json('POST', '/api/cashier/create', $payload);
                // \Log::info('CASHIER PAYLOAD:', $payload);
                // \Log::info('CASHIER WITH VALIDATION', (array) $response);
                $response->assertStatus(200)->assertJson([
                    'success' => true,
                    'data' => [],
                    'message' => 'One last thing. We sent a token to your email. Use that to verify the account you created to complete the sign up process. Thanks.'
                    ]);
            }
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function test_create_cashier_password_not_matching_false(): void
    {
        try {
            Sanctum::actingAs(\App\Models\User::where('email', env('TEST_USEREMAIL'))->first(),['*']);
            $faker = Factory::create();
            $response = $this->json('POST', '/api/cashier/create', [
                'email' => $faker->email,
                'firstname' => $faker->firstName,
                'lastname' => $faker->lastName,
                'password' => 'demopass567',
                'password_confirmation' => 'demopass56',
                'account_validation' => 'no',
                'newsletter_validation' => 'yes',
            ]);

            $response->assertStatus(406)->assertJson([
                'success' => false,
                'message' => 'Check these fields first before completing this request.'
                ]);
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }


    public function test_cashier_list_true(): void
    {
        try {
            Sanctum::actingAs(\App\Models\User::where('email', env('TEST_USEREMAIL'))->first(),['*']);
            $response = $this->json('GET', '/api/cashier/list?draw=3&columns%5B0%5D%5Bdata%5D=email&columns%5B0%5D%5Bname%5D=&columns%5B0%5D%5Bsearchable%5D=true&columns%5B0%5D%5Borderable%5D=true&columns%5B0%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B0%5D%5Bsearch%5D%5Bregex%5D=false&columns%5B1%5D%5Bdata%5D=role&columns%5B1%5D%5Bname%5D=&columns%5B1%5D%5Bsearchable%5D=true&columns%5B1%5D%5Borderable%5D=true&columns%5B1%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B1%5D%5Bsearch%5D%5Bregex%5D=false&columns%5B2%5D%5Bdata%5D=status&columns%5B2%5D%5Bname%5D=&columns%5B2%5D%5Bsearchable%5D=true&columns%5B2%5D%5Borderable%5D=true&columns%5B2%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B2%5D%5Bsearch%5D%5Bregex%5D=false&columns%5B3%5D%5Bdata%5D=parent_id&columns%5B3%5D%5Bname%5D=&columns%5B3%5D%5Bsearchable%5D=true&columns%5B3%5D%5Borderable%5D=true&columns%5B3%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B3%5D%5Bsearch%5D%5Bregex%5D=false&columns%5B4%5D%5Bdata%5D=created_at&columns%5B4%5D%5Bname%5D=&columns%5B4%5D%5Bsearchable%5D=true&columns%5B4%5D%5Borderable%5D=true&columns%5B4%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B4%5D%5Bsearch%5D%5Bregex%5D=false&columns%5B5%5D%5Bdata%5D=verified_at&columns%5B5%5D%5Bname%5D=&columns%5B5%5D%5Bsearchable%5D=true&columns%5B5%5D%5Borderable%5D=true&columns%5B5%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B5%5D%5Bsearch%5D%5Bregex%5D=false&order%5B0%5D%5Bcolumn%5D=0&order%5B0%5D%5Bdir%5D=asc&start=0&length=10&search%5Bvalue%5D=&search%5Bregex%5D=false', []);
            $response->assertStatus(200)->assertJson([
                'draw' => '3',
                'recordsTotal' => 10,
                'recordsFiltered' => 0,
                'server_cols' => '',
                'data' => []
                ]);


        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function test_cashier_deletion_true(): void
    {
        try {
            Sanctum::actingAs(\App\Models\User::where('email', env('TEST_USEREMAIL'))->first(),['*']);
            $response = $this->json('POST', '/api/cashier/delete', [
                'user_id' => 3
            ]);
            $response->assertStatus(200)->assertJson([
                'success' => true,
                'message' => 'User data successfully deleted.'
                ]);


        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function test_cashier_deletion_false(): void
    {
        try {
            Sanctum::actingAs(\App\Models\User::where('email', env('TEST_USEREMAIL'))->first(),['*']);
            $response = $this->json('POST', '/api/cashier/delete', [
                'user_id' => 3
            ]);

            $response->assertStatus(406)->assertJson([
                'success' => false,
                'message' => 'Cashier data not found.'
                ]);
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
}
