<?php

namespace Tests\Feature\Main;

use Faker\Factory;
use Faker\Generator;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use Tests\TestCase;
use Laravel\Sanctum\Sanctum;

class NotificationTest extends TestCase
{
    //Test simple notification true
    public function test_create_simple_notification_true(): void
    {
        try {
            Sanctum::actingAs(\App\Models\User::where('email', env('TEST_USEREMAIL'))->first(),['*']);
            for ($i = 0; $i < 5; $i++) {
                $user = \App\Models\User::inRandomOrder()->where('id', '>', 1)->limit(1)->get();
                $faker = Factory::create();
                //Create 5 txn

                $response = $this->json('POST', '/api/notification/create', [
                    'promo_name' => $faker->catchPhrase,
                    'promo_details' => $faker->realText(200, 2),
                    'rule' => $faker->realText(200, 2),
                    'format' => 'email',
                    'start' => date('Y-m-d H:i:s'),
                    'end' => date('Y-m-d H:i:s', strtotime('+2 months')),
                    'broadcast' => 'yes'
                ]);

                $response->assertStatus(200)->assertJson([
                    'success' => true,
                    // 'data' => ['txn' => $transaction->toArray()],
                    'message' => 'Notification data has been created.'
                    ]);
            }
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
    //Test duplicate notification true
    public function test_duplicate_simple_notification_true(): void
    {
        try {
            Sanctum::actingAs(\App\Models\User::where('email', env('TEST_USEREMAIL'))->first(),['*']);
            for ($i = 0; $i < 5; $i++) {
                $notif = \App\Models\Notification::inRandomOrder()->where('id', '>', 1)->limit(1)->get();
                $faker = Factory::create();
                //Create 5 txn

                $response = $this->json('POST', '/api/notification/create', [
                    'promo_name' => $notif[0]->promo_name,
                    'promo_details' => $faker->realText(200, 2),
                    'rule' => $faker->realText(200, 2),
                    'format' => 'email',
                    'start' => date('Y-m-d H:i:s'),
                    'end' => date('Y-m-d H:i:s', strtotime('+2 months')),
                    'broadcast' => 'no'
                ]);

                $response->assertStatus(200)->assertJson([
                    'success' => true,
                    // 'data' => ['txn' => $transaction->toArray()],
                    'message' => 'Notification data has been created.'
                    ]);
            }
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
    //Test notification format true (email / platform)
    public function test_notification_format_true(): void
    {
        try {
            Sanctum::actingAs(\App\Models\User::where('email', env('TEST_USEREMAIL'))->first(),['*']);
            for ($i = 0; $i < 2; $i++) {
                $format = $i == 0 ? 'email' : 'platform';
                $faker = Factory::create();
                $response = $this->json('POST', '/api/notification/create', [
                    'promo_name' => $faker->catchPhrase,
                    'promo_details' => $faker->realText(200, 2),
                    'rule' => $faker->realText(200, 2),
                    'format' => $format,
                    'start' => date('Y-m-d H:i:s'),
                    'end' => date('Y-m-d H:i:s', strtotime('+2 months')),
                    'broadcast' => 'yes'
                ]);

                $response->assertStatus(200)->assertJson([
                    'success' => true,
                    // 'data' => ['txn' => $transaction->toArray()],
                    'message' => 'Notification data has been created.'
                    ]);
            }
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
    //Test notification format false
    public function test_notification_format_false(): void
    {
        try {
            Sanctum::actingAs(\App\Models\User::where('email', env('TEST_USEREMAIL'))->first(),['*']);
            for ($i = 0; $i < 2; $i++) {
                $format = 'phone';
                $faker = Factory::create();
                $response = $this->json('POST', '/api/notification/create', [
                    'promo_name' => $faker->catchPhrase,
                    'promo_details' => $faker->realText(200, 2),
                    'rule' => $faker->realText(200, 2),
                    'format' => $format,
                    'start' => date('Y-m-d H:i:s'),
                    'end' => date('Y-m-d H:i:s', strtotime('+2 months')),
                    'broadcast' => 'no'
                ]);

                $response->assertStatus(200)->assertJson([
                    'success' => true,
                    // 'data' => ['txn' => $transaction->toArray()],
                    'message' => 'Notification data has been created.'
                    ]);
            }
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
    //Test notification list true
    public function test_customer_list_true(): void
    {
        try {
            Sanctum::actingAs(\App\Models\User::where('email', env('TEST_USEREMAIL'))->first(),['*']);
            $response = $this->json('GET', '/api/notification/list?draw=3&columns%5B0%5D%5Bdata%5D=promo_name&columns%5B0%5D%5Bname%5D=&columns%5B0%5D%5Bsearchable%5D=true&columns%5B0%5D%5Borderable%5D=true&columns%5B0%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B0%5D%5Bsearch%5D%5Bregex%5D=false&columns%5B1%5D%5Bdata%5D=promo_details&columns%5B1%5D%5Bname%5D=&columns%5B1%5D%5Bsearchable%5D=true&columns%5B1%5D%5Borderable%5D=true&columns%5B1%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B1%5D%5Bsearch%5D%5Bregex%5D=false&columns%5B2%5D%5Bdata%5D=format&columns%5B2%5D%5Bname%5D=&columns%5B2%5D%5Bsearchable%5D=true&columns%5B2%5D%5Borderable%5D=true&columns%5B2%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B2%5D%5Bsearch%5D%5Bregex%5D=false&columns%5B3%5D%5Bdata%5D=rule&columns%5B3%5D%5Bname%5D=&columns%5B3%5D%5Bsearchable%5D=true&columns%5B3%5D%5Borderable%5D=true&columns%5B3%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B3%5D%5Bsearch%5D%5Bregex%5D=false&columns%5B4%5D%5Bdata%5D=start&columns%5B4%5D%5Bname%5D=&columns%5B4%5D%5Bsearchable%5D=true&columns%5B4%5D%5Borderable%5D=true&columns%5B4%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B4%5D%5Bsearch%5D%5Bregex%5D=false&columns%5B5%5D%5Bdata%5D=end&columns%5B5%5D%5Bname%5D=&columns%5B5%5D%5Bsearchable%5D=true&columns%5B5%5D%5Borderable%5D=true&columns%5B5%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B5%5D%5Bsearch%5D%5Bregex%5D=false&order%5B0%5D%5Bcolumn%5D=0&order%5B0%5D%5Bdir%5D=asc&start=0&length=10&search%5Bvalue%5D=&search%5Bregex%5D=false', []);
            $response->assertStatus(200)->assertJson([
                'draw' => '3',
                'recordsTotal' => 7,
                'recordsFiltered' => 0,
                'server_cols' => '',
                'data' => []
                ]);


        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
    //Test notification deletion
    public function test_notification_deletion_true(): void
    {
        try {
            Sanctum::actingAs(\App\Models\User::where('email', env('TEST_USEREMAIL'))->first(),['*']);
            $response = $this->json('POST', '/api/notification/delete', [
                'notif_id' => 5
            ]);
            $response->assertStatus(200)->assertJson([
                'success' => true,
                'message' => 'Transaction data is successfully deleted.'
                ]);


        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
}
