<?php

namespace Tests\Feature\Main;

use Faker\Factory;
use Faker\Generator;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use Tests\TestCase;
use Laravel\Sanctum\Sanctum;

class ProfileTest extends TestCase
{
    //Update first name and lastname same user
    public function test_same_user_firstname_lastname_true(): void
    {
        try {
            $user = \App\Models\User::where('email', env('TEST_USEREMAIL'))->first();
            Sanctum::actingAs($user,['*']);
            $faker = Factory::create();
            $response = $this->json('POST', '/api/user/update', [
                'user_id' => $user->id,
                'firstname' => $faker->firstName,
                'lastname' => $faker->lastName,
                'newsletter_validation' => 'yes'
            ]);
            $response->assertStatus(200)->assertJson([
                'success' => true,
                'message' => 'User data has been updated.'
                ]);
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    //Update first name and lastname different user but admin
    public function test_different_user_but_admin_false(): void
    {
        try {
            $admin = \App\Models\User::where('email', env('TEST_USEREMAIL'))->first();
            Sanctum::actingAs($admin,['*']);
            $user = \App\Models\User::inRandomOrder()->where('id', '>', 1)->limit(1)->get();
            $faker = Factory::create();
            $response = $this->json('POST', '/api/user/update', [
                'user_id' => $user[0]->id,
                'firstname' => $faker->firstName,
                'lastname' => $faker->lastName,
                'newsletter_validation' => 'no'
            ]);
            $response->assertStatus(200)->assertJson([
                'success' => true,
                'message' => 'User data has been updated.'
                ]);
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    //Update first name and lastname different user non admin
    public function test_different_user_non_admin_false(): void
    {
        try {
            $nonadmin = \App\Models\User::inRandomOrder()->where('id', '>', 1)->limit(1)->get();
            Sanctum::actingAs($nonadmin[0],['*']);
            $user = \App\Models\User::inRandomOrder()->where('id', '>', 1)->limit(1)->get();
            $faker = Factory::create();
            $response = $this->json('POST', '/api/user/update', [
                'user_id' => $user[0]->id,
                'firstname' => $faker->firstName,
                'lastname' => $faker->lastName,
                'newsletter_validation' => 'no'
            ]);
            $response->assertStatus(406)->assertJson([
                'success' => false,
                'message' => 'Update request is not allowed.'
                ]);
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    //Update display name
    public function test_update_display_name_true(): void
    {
        try {
            $user = \App\Models\User::where('email', env('TEST_USEREMAIL'))->first();
            Sanctum::actingAs($user,['*']);
            $faker = Factory::create();
            $response = $this->json('POST', '/api/user/update', [
                'user_id' => $user->id,
                'firstname' => $faker->firstName,
                'lastname' => $faker->lastName,
                'display_name' => $faker->lastName,
                'newsletter_validation' => 'no'
            ]);
            $response->assertStatus(200)->assertJson([
                'success' => true,
                'message' => 'User data has been updated.'
                ]);
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
    //Update mobile
    public function test_update_mobile_true(): void
    {
        try {
            $user = \App\Models\User::where('email', env('TEST_USEREMAIL'))->first();
            Sanctum::actingAs($user,['*']);
            $faker = Factory::create();
            $response = $this->json('POST', '/api/user/update', [
                'user_id' => $user->id,
                'firstname' => $faker->firstName,
                'lastname' => $faker->lastName,
                'mobile' => '63917123456',
                'newsletter_validation' => 'yes'
            ]);
            $response->assertStatus(200)->assertJson([
                'success' => true,
                'message' => 'User data has been updated.'
                ]);
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
    //Update avatar
    // public function test_upload_avatar_true(): void
    // {
    //     try {
    //         $user = \App\Models\User::where('email', env('TEST_USEREMAIL'))->first();
    //         Sanctum::actingAs($user,['*']);
    //         $faker = Factory::create();
    //         $response = $this->json('POST', '/api/user/update', [
    //             'user_id' => $user->id,
    //             'firstname' => $faker->firstName,
    //             'lastname' => $faker->lastName,
    //             'mobile' => '63917123456'
    //         ]);
    //         $response->assertStatus(200)->assertJson([
    //             'success' => true,
    //             'message' => 'User data has been updated.'
    //             ]);
    //     } catch (\Exception $e) {
    //         \Log::error($e);
    //     }
    // }
    //Update status (active)
    public function test_update_status_active_true(): void
    {
        try {
            $user = \App\Models\User::where('email', env('TEST_USEREMAIL'))->first();
            Sanctum::actingAs($user,['*']);
            $faker = Factory::create();
            $response = $this->json('POST', '/api/user/update', [
                'user_id' => $user->id,
                'firstname' => $faker->firstName,
                'lastname' => $faker->lastName,
                'status' => 'active',
                'newsletter_validation' => 'yes'
            ]);
            $response->assertStatus(200)->assertJson([
                'success' => true,
                'message' => 'User data has been updated.'
                ]);
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
    //Update status (inactive)
    public function test_update_status_inactive_true(): void
    {
        try {
            $user = \App\Models\User::where('email', env('TEST_USEREMAIL'))->first();
            Sanctum::actingAs($user,['*']);
            $faker = Factory::create();
            $response = $this->json('POST', '/api/user/update', [
                'user_id' => $user->id,
                'firstname' => $faker->firstName,
                'lastname' => $faker->lastName,
                'status' => 'inactive',
                'newsletter_validation' => 'yes'
            ]);
            $response->assertStatus(200)->assertJson([
                'success' => true,
                'message' => 'User data has been updated.'
                ]);
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

}
