<?php

namespace Tests\Feature\Main;

use Faker\Factory;
use Faker\Generator;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use Tests\TestCase;
use Laravel\Sanctum\Sanctum;

class TransactionTest extends TestCase
{
    public function test_create_transaction_with_points_true(): void
    {
        try {
            Sanctum::actingAs(\App\Models\User::where('email', env('TEST_USEREMAIL'))->first(),['*']);
            for ($i = 0; $i < 10; $i++) {
                $user = \App\Models\User::inRandomOrder()->where('id', '>', 1)->limit(1)->get();
                $faker = Factory::create();
                //Create 5 txn
                $response = $this->json('POST', '/api/transaction/create', [
                    'customer_id' => $user[0]->id,
                    'service' => $faker->jobTitle,
                    'points' => 1,
                ]);

                $transaction = \App\Models\Transaction::latest()->with('user','cashier')->first();
                $response->assertStatus(200)->assertJson([
                    'success' => true,
                    // 'data' => ['txn' => $transaction->toArray()],
                    'message' => 'Transaction data has been created.'
                    ]);
            }
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function test_create_customer_with_validation_true(): void
    {
        try {
            Sanctum::actingAs(\App\Models\User::where('email', env('TEST_USEREMAIL'))->first(),['*']);
            for ($i = 0; $i < 5; $i++) {
                $user = \App\Models\User::inRandomOrder()->where('id', '>', 1)->limit(1)->get();
                $faker = Factory::create();
                //Create 5 txn
                $response = $this->json('POST', '/api/transaction/create', [
                    'customer_id' => $user[0]->id,
                    'service' => $faker->jobTitle,
                ]);

                $transaction = \App\Models\Transaction::latest()->with('user','cashier')->first();
                $response->assertStatus(200)->assertJson([
                    'success' => true,
                    // 'data' => ['txn' => $transaction->toArray()],
                    'message' => 'Transaction data has been created.'
                    ]);
            }
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }


    public function test_customer_list_true(): void
    {
        try {
            Sanctum::actingAs(\App\Models\User::where('email', env('TEST_USEREMAIL'))->first(),['*']);
            $response = $this->json('GET', '/api/transaction/list?draw=3&columns%5B0%5D%5Bdata%5D=email&columns%5B0%5D%5Bname%5D=&columns%5B0%5D%5Bsearchable%5D=true&columns%5B0%5D%5Borderable%5D=true&columns%5B0%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B0%5D%5Bsearch%5D%5Bregex%5D=false&columns%5B1%5D%5Bdata%5D=role&columns%5B1%5D%5Bname%5D=&columns%5B1%5D%5Bsearchable%5D=true&columns%5B1%5D%5Borderable%5D=true&columns%5B1%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B1%5D%5Bsearch%5D%5Bregex%5D=false&columns%5B2%5D%5Bdata%5D=status&columns%5B2%5D%5Bname%5D=&columns%5B2%5D%5Bsearchable%5D=true&columns%5B2%5D%5Borderable%5D=true&columns%5B2%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B2%5D%5Bsearch%5D%5Bregex%5D=false&columns%5B3%5D%5Bdata%5D=parent_id&columns%5B3%5D%5Bname%5D=&columns%5B3%5D%5Bsearchable%5D=true&columns%5B3%5D%5Borderable%5D=true&columns%5B3%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B3%5D%5Bsearch%5D%5Bregex%5D=false&columns%5B4%5D%5Bdata%5D=created_at&columns%5B4%5D%5Bname%5D=&columns%5B4%5D%5Bsearchable%5D=true&columns%5B4%5D%5Borderable%5D=true&columns%5B4%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B4%5D%5Bsearch%5D%5Bregex%5D=false&columns%5B5%5D%5Bdata%5D=verified_at&columns%5B5%5D%5Bname%5D=&columns%5B5%5D%5Bsearchable%5D=true&columns%5B5%5D%5Borderable%5D=true&columns%5B5%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B5%5D%5Bsearch%5D%5Bregex%5D=false&order%5B0%5D%5Bcolumn%5D=0&order%5B0%5D%5Bdir%5D=asc&start=0&length=10&search%5Bvalue%5D=&search%5Bregex%5D=false', []);
            $response->assertStatus(200)->assertJson([
                'draw' => '3',
                'recordsTotal' => 10,
                'recordsFiltered' => 0,
                'server_cols' => '',
                'data' => []
                ]);


        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function test_transaction_deletion_true(): void
    {
        try {
            Sanctum::actingAs(\App\Models\User::where('email', env('TEST_USEREMAIL'))->first(),['*']);
            $response = $this->json('POST', '/api/transaction/delete', [
                'txn_id' => 8
            ]);
            $response->assertStatus(200)->assertJson([
                'success' => true,
                'message' => 'Transaction data is successfully deleted.'
                ]);


        } catch (\Exception $e) {
            \Log::error($e);
        }
    }

    public function test_transaction_deletion_false(): void
    {
        try {
            Sanctum::actingAs(\App\Models\User::where('email', env('TEST_USEREMAIL'))->first(),['*']);
            $response = $this->json('POST', '/api/transaction/delete', [
                'txn_id' => 8
            ]);

            $response->assertStatus(406)->assertJson([
                'success' => false,
                'message' => 'Transaction data not found.'
                ]);
        } catch (\Exception $e) {
            \Log::error($e);
        }
    }
}
